@@ -61,7 +61,7 @@ void    (*read_apm_data)();
 
 /* ------- FreeBSD ------------------------------------------------------- */
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) && defined(__i386__)
 #include <osreldate.h>
 #include <machine/apm_bios.h>
 #define	APMDEV		"/dev/apm"
@@ -141,22 +141,23 @@ read_linux_apm_data(void)
 
 void
 read_netbsd_apm_data()
-{
-   int f, r;
-   struct apm_power_info info;
+	{
+	int f, r;
+	struct apm_power_info info;
 
-   if ((f = open(APMDEV, O_RDONLY)) == -1) return;
-   r = ioctl(f, APM_IOC_GETPOWER, &info);
-   close(f);
-   if (r == -1) return;
+	if ((f = open(APMDEV, O_RDONLY)) == -1) return;
+	memset(&info, 0, sizeof(info));
+	r = ioctl(f, APM_IOC_GETPOWER, &info);
+	close(f);
+	if (r == -1) return;
 
-   sys_apm.battery_is_available = (info.battery_state != APM_BATT_UNKNOWN);
-   sys_apm.ac_is_on_line = (info.ac_state == APM_AC_ON) ? TRUE : FALSE;
-   sys_apm.battery_is_charging = 
-      (info.battery_state == APM_BATT_CHARGING) ? TRUE : FALSE;
-   sys_apm.battery_percentage = info.battery_life;
-   sys_apm.battery_time_left = info.minutes_left;
-}
+	sys_apm.battery_is_available = (info.battery_state != APM_BATT_UNKNOWN);
+	sys_apm.ac_is_on_line = (info.ac_state == APM_AC_ON) ? TRUE : FALSE;
+	sys_apm.battery_is_charging = 
+		(info.battery_state == APM_BATT_CHARGING) ? TRUE : FALSE;
+	sys_apm.battery_percentage = info.battery_life;
+	sys_apm.battery_time_left = info.minutes_left;
+	}
 
 #endif /* __NetBSD__ || __OpenBSD__ */
 
@@ -206,7 +207,7 @@ setup_apm_interface(void)
 		return TRUE;
 		}
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) && defined(__i386__)
 	read_apm_data = read_freebsd_apm_data;
 	available = TRUE;
 #endif
@@ -373,22 +374,23 @@ static void
 create_apm(GtkWidget *vbox, gint first_create)
 	{
 	Style		*style;
+	Margin		*m;
 	gint		x, w;
 
 	if (first_create)
 		apm = gkrellm_panel_new0();
 
 	style = gkrellm_meter_style(style_id);
-
+	m = gkrellm_get_style_margins(style);
 	power_source_decal = gkrellm_create_decal_pixmap(apm, GK.decal_misc_pixmap,
-			GK.decal_misc_mask, N_MISC_DECALS, style, style->margin + 2, -1);
+			GK.decal_misc_mask, N_MISC_DECALS, style, m->left + 2, -1);
 
 	/* The default text decal has width of chart_width - margins and
 	|  x position at left margin, so calculate the override values.
 	|  Cannot override w after the create.
 	*/
-	x = style->margin + power_source_decal->w + 6;
-	w = gkrellm_chart_width() - x - style->margin;
+	x = m->left + power_source_decal->w + 6;
+	w = gkrellm_chart_width() - x - (m->left + m->right);
 	minutes_left_decal = gkrellm_create_decal_text(apm, "100",
 						gkrellm_meter_textstyle(style_id),
 						style, x, -1, w);	/* -1 means use y default */
