@@ -52,7 +52,11 @@ typedef struct
 static gboolean		cal_enable,
 					clock_enable,
 					clock_12hr_seconds,
-					clock_24hr_format;
+					clock_24hr_seconds,
+					clock_24hr_mode;
+
+static gint			x_clock_seconds,
+					x_clock_no_seconds;
 
 static Extents		wday_extents,
 					mday_extents,
@@ -113,7 +117,7 @@ draw_clock(struct tm *t, gboolean update_minutes)
 		return;
 	if (update_minutes)
 		{
-		if (clock_24hr_format)
+		if (clock_24hr_mode)
 			strftime(buf, sizeof(buf), "%k:%M", t);   /* 24 hour:minute */
 		else
 			strftime(buf, sizeof(buf), "%l:%M", t);   /* 12 hour:minute */
@@ -121,12 +125,14 @@ draw_clock(struct tm *t, gboolean update_minutes)
 		d_clock->x_off = (w < d_clock->w) ? (d_clock->w - w) / 2 : 0;
 		gkrellm_draw_decal_text(pclock, d_clock, buf, -1);
 		}
-	if (clock_12hr_seconds || clock_24hr_format)
+	if (   (!clock_24hr_mode && clock_12hr_seconds)
+		|| (clock_24hr_mode && clock_24hr_seconds)
+	   )
 		{
 		strftime(sec_ampm, sizeof(sec_ampm), "%S", t);	/* seconds 00-59 */
 		gkrellm_draw_decal_text(pclock, d_seconds, sec_ampm, -1);
 		}
-	else
+	else if (! clock_24hr_mode)
 		{
 		strftime(sec_ampm, sizeof(sec_ampm), "%p", t);	/* AM/PM */
 		gkrellm_draw_decal_text(pclock, d_seconds, sec_ampm, -1);
@@ -238,6 +244,23 @@ create_calendar(GtkWidget *vbox, gint first_create)
 	draw_cal(&gkrellm_current_tm);
 	}
 
+static void
+set_seconds_display()
+	{
+	if (   !clock_24hr_mode
+		|| (clock_24hr_mode && clock_24hr_seconds)
+	   )
+		{
+		gkrellm_move_decal(pclock, d_clock, x_clock_seconds, d_clock->y);
+		gkrellm_make_decal_visible(pclock, d_seconds);
+		}
+	else
+		{
+		gkrellm_move_decal(pclock, d_clock, x_clock_no_seconds, d_clock->y);
+		gkrellm_make_decal_invisible(pclock, d_seconds);
+		}
+	}
+
 static void
 create_clock(GtkWidget *vbox, gint first_create)
 	{
@@ -259,8 +282,9 @@ create_clock(GtkWidget *vbox, gint first_create)
 	d_seconds = gkrellm_create_decal_text(pclock, "88", &sec_extents.ts,
 				style, 0, -1, sec_extents.width + 2);
 
-	d_clock->x = (gkrellm_chart_width() - d_clock->w - d_seconds->w) / 2;
-	d_seconds->x = d_clock->x + d_clock->w + 2;
+	x_clock_seconds = (gkrellm_chart_width() - d_clock->w - d_seconds->w) / 2;
+	x_clock_no_seconds = (gkrellm_chart_width() - d_clock->w) / 2;
+	d_seconds->x = x_clock_seconds + d_clock->w + 2;
 	d_seconds->y = d_clock->y + d_clock->h - d_seconds->h;
 
 	gkrellm_panel_configure(pclock, NULL, style);
@@ -275,12 +299,12 @@ create_clock(GtkWidget *vbox, gint first_create)
 		}
 	gkrellm_setup_launcher(pclock, &clock_launch, METER_PANEL_TYPE, 0);
 
+	set_seconds_display();
 	if (!clock_enable)
 		{
 		gkrellm_panel_hide(pclock);
 		hide_spacers(mon_clock);
 		}
-
 	draw_clock(&gkrellm_current_tm, TRUE);
 	}
 
@@ -297,8 +321,9 @@ save_clock_cal_config(FILE *f)
 				cal_launch.command);
 	fprintf(f, "%s cal_tooltip %s\n", CLOCK_CONFIG_KEYWORD,
 				cal_launch.tooltip_comment);
-	fprintf(f, "%s clock_options %d %d %d\n", CLOCK_CONFIG_KEYWORD,
-				clock_enable, clock_12hr_seconds, clock_24hr_format);
+	fprintf(f, "%s clock_options %d %d %d %d\n", CLOCK_CONFIG_KEYWORD,
+				clock_enable, clock_12hr_seconds, clock_24hr_mode, 
+				clock_24hr_seconds);
 	fprintf(f, "%s cal_options %d\n", CLOCK_CONFIG_KEYWORD,
 				cal_enable);
 	}
@@ -321,8 +346,8 @@ load_clock_cal_config(gchar *arg)
 		else if (!strcmp(config, "cal_tooltip"))
 			cal_launch.tooltip_comment = g_strdup(item);
 		else if (!strcmp(config, "clock_options"))
-			sscanf(item, "%d %d %d", &clock_enable, &clock_12hr_seconds,
-					&clock_24hr_format);
+			sscanf(item, "%d %d %d %d", &clock_enable, &clock_12hr_seconds,
+					&clock_24hr_mode, &clock_24hr_seconds);
 		else if (!strcmp(config, "cal_options"))
 			sscanf(item, "%d", &cal_enable);
 		}
@@ -336,14 +361,16 @@ static GtkWidget	*cal_launch_entry,
 					*enable_clk_button,
 					*enable_cal_button,
 					*clk_24hr_button,
+					*clk_24hr_sec_button,
 					*clk_12hr_sec_button;
 
 
 static void
 apply_clock_cal_config()
 	{
-	clock_24hr_format = GTK_TOGGLE_BUTTON (clk_24hr_button)->active;
+	clock_24hr_mode = GTK_TOGGLE_BUTTON (clk_24hr_button)->active;
 	clock_12hr_seconds = GTK_TOGGLE_BUTTON (clk_12hr_sec_button)->active;
+	clock_24hr_seconds = GTK_TOGGLE_BUTTON (clk_24hr_sec_button)->active;
 
 	clock_enable = GTK_TOGGLE_BUTTON (enable_clk_button)->active;
 	if (clock_enable)
@@ -368,16 +395,36 @@ apply_clock_cal_config()
 	gkrellm_apply_launcher(&clock_launch_entry, &clock_tooltip_entry, pclock,
 					&clock_launch, gkrellm_launch_button_cb);
 
+	set_seconds_display();
 	draw_cal(&gkrellm_current_tm);
 	draw_clock(&gkrellm_current_tm, TRUE);
 	}
 
+static void
+set_seconds_buttons_sensitivity(gboolean mode_24hr)
+	{
+	if (mode_24hr)
+		{
+		gtk_widget_set_sensitive(clk_24hr_sec_button, TRUE);
+		gtk_widget_set_sensitive(clk_12hr_sec_button, FALSE);
+		}
+	else
+		{
+		gtk_widget_set_sensitive(clk_24hr_sec_button, FALSE);
+		gtk_widget_set_sensitive(clk_12hr_sec_button, TRUE);
+		}
+	}
+
+static void cb_clk_24hr(GtkToggleButton *tb, gpointer user_data)
+	{
+	set_seconds_buttons_sensitivity(tb->active);
+	}
 
 static void
 create_clock_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget	*tabs;
-	GtkWidget	*table, *vbox, *vbox1;
+	GtkWidget	*table, *vbox, *vbox1, *hbox;
 
 	tabs = gtk_notebook_new();
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
@@ -390,14 +437,25 @@ create_clock_tab(GtkWidget *tab_vbox)
 				_("Enable"));
 
 	vbox1 = gkrellm_framed_vbox_end(vbox, _("Clock"), 4, FALSE, 0, 2);
-	gkrellm_check_button(vbox1, &enable_clk_button, clock_enable, FALSE, 2,
-				_("Enable"));
 
-	gkrellm_check_button(vbox1, &clk_12hr_sec_button, clock_12hr_seconds, FALSE,
-			0, _("Show seconds instead of am/pm in 12 hour time mode"));
+	gkrellm_check_button(vbox1, &enable_clk_button, clock_enable, 
+			FALSE, 6, _("Enable"));
 
-	gkrellm_check_button(vbox1, &clk_24hr_button, clock_24hr_format, FALSE, 0,
-					_("Display 24 hour instead of 12 hour time"));
+	gkrellm_check_button_connected(vbox1, &clk_24hr_button, clock_24hr_mode,
+			FALSE, FALSE, 0, &cb_clk_24hr, 0, 
+			_("Display 24 hour instead of 12 hour time"));
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	gkrellm_check_button(hbox, &clk_12hr_sec_button, clock_12hr_seconds,
+			FALSE,20, _("Show seconds instead of am/pm in 12 hour time mode"));
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	gkrellm_check_button(hbox, &clk_24hr_sec_button, clock_24hr_seconds,
+			FALSE, 20, _("Show seconds in 24 hour time mode"));
+
+	set_seconds_buttons_sensitivity(clock_24hr_mode);
 
 /* -- Setup tab */
 	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
@@ -441,6 +499,7 @@ init_clock_monitor(void)
 	clock_enable = TRUE;
     monitor_clock.name = _(monitor_clock.name);
 	clock_12hr_seconds = TRUE;
+	clock_24hr_seconds = TRUE;
 	clock_style_id = gkrellm_add_meter_style(&monitor_clock, CLOCK_STYLE_NAME);
 	mon_clock = &monitor_clock;
 	return &monitor_clock;
