@@ -175,18 +175,38 @@ gkrellm_theme_file_exists(char *name, gchar *subdir)
 static void
 set_border(GdkImlibBorder *border, char *string)
 	{
-	if (border == NULL)
+	if (!border)
 		return;
-	border->top = 0;
-	border->bottom = 0;
 	border->left = 0;
 	border->right = 0;
+	border->top = 0;
+	border->bottom = 0;
 	if (string == NULL)
 		return;
 	sscanf(string, "%d,%d,%d,%d", &border->left, &border->right,
 				&border->top, &border->bottom);
 	}
 
+static void
+set_margins(Style *style, char *string)
+	{
+	Margin	*margins;
+
+	GK.use_top_bottom_margins = TRUE;
+	if (!style)
+		return;
+	margins = &style->margins;
+	margins->left = 0;
+	margins->right = 0;
+	margins->top = 0;
+	margins->bottom = 0;
+	if (string == NULL)
+		return;
+	sscanf(string, "%d,%d,%d,%d", &margins->left, &margins->right,
+				&margins->top, &margins->bottom);
+	style->margin = margins->left;	/* Compatibility */
+	}
+
 
 static void
 assign_font(Style *style, gchar *fontname, gint AorB)
@@ -228,6 +248,20 @@ assign_textcolor(Style *style, gchar *arg, gint AorB)
 	g_free(values);
 	}
 
+void
+gkrellm_set_style_krell_values(Style *s, gint yoff, gint depth, gint x_hot,
+				gint expand, gint left_margin, gint right_margin)
+	{
+	if (!s)
+		return;
+	s->krell_yoff = yoff;
+	s->krell_depth = depth;
+	s->krell_x_hot = x_hot;
+	s->krell_expand = expand;
+	s->krell_left_margin = left_margin;
+	s->krell_right_margin = right_margin;
+	}
+
 void
 gkrellm_set_krell_expand(Style *style, gchar *value)
 	{
@@ -239,10 +273,16 @@ gkrellm_set_krell_expand(Style *style, gchar *value)
 		{
 		if (!strcmp(value, "left"))
 			expand = KRELL_EXPAND_LEFT;
+		else if (!strcmp(value, "left-scaled"))
+			expand = KRELL_EXPAND_LEFT_SCALED;
 		else if (!strcmp(value, "right"))
 			expand = KRELL_EXPAND_RIGHT;
+		else if (!strcmp(value, "right-scaled"))
+			expand = KRELL_EXPAND_RIGHT_SCALED;
 		else if (!strcmp(value, "bar-mode"))
 			expand = KRELL_EXPAND_BAR_MODE;
+		else if (!strcmp(value, "bar-mode-scaled"))
+			expand = KRELL_EXPAND_BAR_MODE_SCALED;
 		}
 	style->krell_expand = expand;
 	}
@@ -260,27 +300,39 @@ assign_style_entry(Style *style, gchar *value, gint entry_flag)
 		style->krell_ema_period = atoi(value);
 	else if (entry_flag == SET_KRELL_DEPTH)
 		style->krell_depth = atoi(value);
+	else if (entry_flag == SET_KRELL_LEFT_MARGIN)
+		style->krell_left_margin = atoi(value);
+	else if (entry_flag == SET_KRELL_RIGHT_MARGIN)
+		style->krell_right_margin = atoi(value);
 	else if (entry_flag == SET_LABEL_POSITION)
 		{
 		if (strcmp(value, "center") == 0)
-			style->label_position = LABEL_CENTER;
+			style->label_position = GKRELLM_LABEL_CENTER;
 		else if (isdigit(*value))
 			style->label_position = atoi(value);
 		else
-			style->label_position = LABEL_NONE;
+			style->label_position = GKRELLM_LABEL_NONE;
 		}
-	else if (entry_flag == SET_MARGIN)
+	else if (entry_flag == SET_MARGIN)	/* Deprecated as of 1.2.9 */
+		{
 		style->margin = atoi(value);
+		style->margins.left = style->margin;
+		style->margins.right = style->margin;
+		}
 	else if (entry_flag == SET_TOP_MARGIN)
 		{
-		style->top_margin = atoi(value);
+		style->margins.top = atoi(value);
 		GK.use_top_bottom_margins = TRUE;	/* Allow themes to adapt. */
 		}
 	else if (entry_flag == SET_BOTTOM_MARGIN)
 		{
-		style->bottom_margin = atoi(value);
+		style->margins.bottom = atoi(value);
 		GK.use_top_bottom_margins = TRUE;	/* Allow themes to adapt. */
 		}
+	else if (entry_flag == SET_LEFT_MARGIN)
+		style->margins.left = atoi(value);
+	else if (entry_flag == SET_RIGHT_MARGIN)
+		style->margins.right = atoi(value);
 	else if (entry_flag == SET_TRANSPARENCY)
 		style->transparency = atoi(value);
 	else if (entry_flag == SET_TEXTCOLOR_A)
@@ -293,12 +345,14 @@ assign_style_entry(Style *style, gchar *value, gint entry_flag)
 		assign_font(style, value, SET_TEXTFONT_B);
 	else if (entry_flag == SET_BORDER)
 		set_border(&style->border, value);
+	else if (entry_flag == SET_MARGINS)
+		set_margins(style, value);
 	}
 
 
 static void
 assign_style(gchar *debug_name, GList *style_list, gint index,
-				gchar *arg, gint entry_flag, gint override)
+				gchar *arg, gint entry_value, gint override)
 	{
 	Style	*style;
 	GList	*list;
@@ -310,48 +364,74 @@ assign_style(gchar *debug_name, GList *style_list, gint index,
 	/* If this is not an override assignment and this entry has already had
 	|  an override assignment, then we do not assign.
 	*/
-	if (! override && (style->override & BIT(entry_flag)))
+	if (! override && (style->override & BIT(entry_value)))
 		return;
 	if (override)
-		style->override |= BIT(entry_flag);
-	assign_style_entry(style, arg, entry_flag);
+		style->override |= BIT(entry_value);
+	assign_style_entry(style, arg, entry_value);
 	if (index > 0)	/* Theme has custom setting for this style */
 		style->themed = TRUE;
 	if (index++ == 0)		/* style == style_list */
 		{
 		if (override)
 			printf("Bad override on DEFAULT: %s %s %d\n",
-					debug_name, arg, entry_flag);
+					debug_name, arg, entry_value);
 		for (list = style_list->next; list; list = list->next, ++index)
 			{
 			style = (Style *) list->data;
-			if (style && !(style->override & BIT(entry_flag)))
-				assign_style_entry(style, arg, entry_flag);
+			if (style && !(style->override & BIT(entry_value)))
+				assign_style_entry(style, arg, entry_value);
 			}
 		}
 	}
 
 #if 0
 static void
-assign_chart_style(gint index, gchar *arg, gint entry_flag, gint override)
+assign_chart_style(gint index, gchar *arg, gint entry_value, gint override)
 	{
 	assign_style("StyleChart", GK.chart_style_list, index, arg,
-				entry_flag, override);
+				entry_value, override);
 	}
 
 static void
-assign_panel_style(gint index, gchar *arg, gint entry_flag, gint override)
+assign_panel_style(gint index, gchar *arg, gint entry_value, gint override)
 	{
 	assign_style("StylePanel", GK.panel_style_list, index, arg,
-				entry_flag, override);
+				entry_value, override);
 	}
 #endif
 
 static void
-assign_meter_style(gint index, gchar *arg, gint entry_flag, gint override)
+assign_meter_style(gint index, gchar *arg, gint entry_value, gint override)
 	{
 	assign_style("StyleMeter", GK.meter_style_list, index, arg,
-				entry_flag, override);
+				entry_value, override);
+	}
+
+static void
+assign_custom_style(gchar *debug_name, GList *style_list, gint index,
+				gchar *arg, gint entry_value, gchar *custom_name)
+	{
+	Style	*style, *custom_style;
+	gint	i;
+
+	if ((i = string_position_in_list(GK.custom_name_list, custom_name)) < 0)
+		{
+		if ((style = (Style *) g_list_nth_data(style_list, index)) == NULL)
+			return;
+		custom_style = gkrellm_copy_style(style);
+		custom_style->themed = TRUE;
+		GK.custom_name_list = g_list_append(GK.custom_name_list,
+				g_strdup(custom_name));
+		GK.custom_style_list = g_list_append(GK.custom_style_list,
+				custom_style);
+		}
+	else
+		custom_style = (Style *) g_list_nth_data(GK.custom_style_list, i);
+
+//printf("assign_custom_style(%s, %s, %d, %s) %d\n",
+//debug_name, custom_name, entry_value, arg, i);
+	assign_style_entry(custom_style, arg, entry_value);
 	}
 
 static struct string_map
@@ -361,41 +441,73 @@ static struct string_map
 	}
 	entry_map[] =
 	{
-	{ "krell_yoff",		SET_KRELL_YOFF },
-	{ "krell_expand",	SET_KRELL_EXPAND },
-	{ "krell_x_hot",	SET_KRELL_X_HOT },
-	{ "krell_ema_period", SET_KRELL_EMA_PERIOD },
-	{ "krell_depth",	SET_KRELL_DEPTH },
-	{ "label_position",	SET_LABEL_POSITION },
-	{ "margin",			SET_MARGIN },
-	{ "top_margin",		SET_TOP_MARGIN },
-	{ "bottom_margin",	SET_BOTTOM_MARGIN },
-	{ "textcolor",		SET_TEXTCOLOR_A },
-	{ "alt_textcolor",	SET_TEXTCOLOR_B },
-	{ "font",			SET_TEXTFONT_A },
-	{ "alt_font",		SET_TEXTFONT_B },
-	{ "border",			SET_BORDER },
-	{ "transparency",	SET_TRANSPARENCY },
+	{ "krell_yoff",			SET_KRELL_YOFF },
+	{ "krell_expand",		SET_KRELL_EXPAND },
+	{ "krell_x_hot",		SET_KRELL_X_HOT },
+	{ "krell_ema_period",	SET_KRELL_EMA_PERIOD },
+	{ "krell_depth",		SET_KRELL_DEPTH },
+	{ "krell_left_margin",	SET_KRELL_LEFT_MARGIN },
+	{ "krell_right_margin",	SET_KRELL_RIGHT_MARGIN },
+	{ "label_position",		SET_LABEL_POSITION },
+	{ "margin",				SET_MARGIN },		/* Deprecated */
+	{ "margins",			SET_MARGINS },
+	{ "left_margin",		SET_LEFT_MARGIN },
+	{ "right_margin",		SET_RIGHT_MARGIN },
+	{ "top_margin",			SET_TOP_MARGIN },
+	{ "bottom_margin",		SET_BOTTOM_MARGIN },
+	{ "textcolor",			SET_TEXTCOLOR_A },
+	{ "alt_textcolor",		SET_TEXTCOLOR_B },
+	{ "font",				SET_TEXTFONT_A },
+	{ "alt_font",			SET_TEXTFONT_B },
+	{ "border",				SET_BORDER },
+	{ "transparency",		SET_TRANSPARENCY },
 	};
 
+
+static gint
+get_entry_value(gchar *entry)
+	{
+	struct string_map	*sm;
+
+	for (sm = &entry_map[0];
+		sm < &entry_map[sizeof(entry_map) / sizeof(struct string_map)]; ++sm)
+		if (!strcmp(entry, sm->string))
+			return sm->value;
+	return -1;
+	}
+
 static void
 assign_gkrellmrc_style(gchar *source_line, gchar *area, gchar *string)
 	{
-	struct string_map	*sm;
-	GList				*list, *style_list, *name_list;
-	gchar				*target, *arg, *scope, *entry;
-	gint				i, style_id, entry_flag, override;
-
-	target = strtok(string, " \t=:");	/* *.yyy  or xxx.yyy */
-	arg = strtok(NULL, "\n");
-	while (*arg == ' ' || *arg == '\t' || *arg == '=' || *arg == ':')
-		++arg;
-	scope = strtok(target, ".");
-	entry = strtok(NULL, " \n");
+	GList	*style_list = NULL, *name_list = NULL;
+	gchar	*s;
+	gchar	*arg = NULL, *mon_name = NULL, *custom_name = NULL, *entry = NULL;
+	gint	index, entry_value, override;
 
+	/* string starts out in format "*.yyy arg" or "foo.yyy arg"
+	*/
+	mon_name = strtok(string, " \t=:");	/* "*.yyy" or "foo.yyy" */
+	if (mon_name && (arg = strtok(NULL, "\n")) != NULL)	/* arg is "arg" part */
+		{
+		while (*arg == ' ' || *arg == '\t' || *arg == '=' || *arg == ':')
+			++arg;
+		entry = strrchr(mon_name, '.');
+		if (entry)
+			*entry++ = '\0';
+		if ((s = strchr(mon_name, '.')) != NULL)
+			{
+			custom_name = g_strdup(mon_name);
+			*s = '\0';
+			}
+		}
+	if (!mon_name || !entry || !*entry || !arg)
+		{
+		printf("StyleXXX ?: %s\n", source_line);
+		g_free(custom_name);
+		return;
+		}
 	override = TRUE;
-	style_id = -1;
-	entry_flag = -1;
+	entry_value = get_entry_value(entry);
 	if (!strcmp(area, "StyleChart"))
 		{
 		name_list = GK.chart_name_list;
@@ -413,51 +525,50 @@ assign_gkrellmrc_style(gchar *source_line, gchar *area, gchar *string)
 		}
 	else
 		{
-		printf(_("unknown Style in %s\n"), source_line);
+		printf("StyleXXX ?: %s\n", source_line);
+		g_free(custom_name);
 		return;
 		}
-	for (list = name_list, i = 0; list; list = list->next, ++i)
-		if (strcmp(scope, (gchar *) list->data) == 0)
-			{
-			style_id = i;
-			break;
-			}
-	if (style_id == DEFAULT_STYLE_ID)
+	index = string_position_in_list(name_list, mon_name);
+	if (index == DEFAULT_STYLE_ID)
 		override = FALSE;
 
-	for (sm = &entry_map[0];
-		sm < &entry_map[sizeof(entry_map) / sizeof(struct string_map)]; ++sm)
-		if (strcmp(entry, sm->string) == 0)
-			{
-			entry_flag = sm->value;
-			break;
-			}
-
-	if (entry_flag >= 0 && style_id >= 0)
-		assign_style(area, style_list, style_id, arg, entry_flag, override);
+	if (entry_value >= 0 && index >= 0)
+		{
+		if (custom_name)
+			assign_custom_style(area, style_list, index, arg, entry_value,
+					custom_name);
+		else
+			assign_style(area, style_list, index, arg, entry_value, override);
+		}
+	g_free(custom_name);
 	}
 
 gint
 gkrellm_add_chart_style(Monitor *mon, gchar *name)
 	{
-	Style		*style;
+	Style		*panel_style, *chart_style;
 	gint		id;
 	static gint	style_id;
 
+	if (!name)
+		return 0;
 	id = style_id++;
+	chart_style = gkrellm_style_new0();
+	panel_style = gkrellm_style_new0();
 	if (mon)
 		{
 		if (mon->privat == NULL)
 			mon->privat = g_new0(MonPrivate, 1);
+		mon->privat->panel_style = panel_style;
+		mon->privat->chart_style = chart_style;
 		mon->privat->style_name = name;
 		mon->privat->style_type = CHART_PANEL_TYPE;
 		mon->privat->style_id = id;
 		}
 	GK.chart_name_list = g_list_append(GK.chart_name_list, (gchar *) name);
-	style = gkrellm_style_new0();
-	GK.chart_style_list = g_list_append(GK.chart_style_list, style);
-	style = gkrellm_style_new0();
-	GK.panel_style_list = g_list_append(GK.panel_style_list, style);
+	GK.chart_style_list = g_list_append(GK.chart_style_list, chart_style);
+	GK.panel_style_list = g_list_append(GK.panel_style_list, panel_style);
 	GK.bg_chart_image_list = g_list_append(GK.bg_chart_image_list, NULL);
 	GK.bg_grid_image_list = g_list_append(GK.bg_grid_image_list, NULL);
 	GK.bg_panel_image_list = g_list_append(GK.bg_panel_image_list, NULL);
@@ -472,16 +583,19 @@ gkrellm_add_meter_style(Monitor *mon, gchar *name)
 	gint		id;
 	static gint	style_id;
 
+	if (!name)
+		return 0;
 	id = style_id++;
+	style = gkrellm_style_new0();
 	if (mon)
 		{
 		if (mon->privat == NULL)
 			mon->privat = g_new0(MonPrivate, 1);
+		mon->privat->panel_style = style;
 		mon->privat->style_name = name;
 		mon->privat->style_type = METER_PANEL_TYPE;
 		mon->privat->style_id = id;
 		}
-	style = gkrellm_style_new0();
 	GK.meter_name_list = g_list_append(GK.meter_name_list, (gchar *) name);
 	GK.meter_style_list = g_list_append(GK.meter_style_list, style);
 	GK.bg_meter_image_list = g_list_append(GK.bg_meter_image_list, NULL);
@@ -489,6 +603,7 @@ gkrellm_add_meter_style(Monitor *mon, gchar *name)
 	return id;
 	}
 
+
 static void
 set_image_borders_in_list(GList *st_list, GList *im_list, GList *nm_list)
 	{
@@ -911,12 +1026,12 @@ load_theme_images(void)
 		n_default = sizeof(default_theme_images) / sizeof(ImageTable);
 		load_image_table(&default_theme_images[0], n_default, NULL);
 
-		if (GK.theme_alternative == 2)
+		if (GK.theme_alternative == 2 || GK.theme_alternative == 5)
 			{
 			n_default = sizeof(default_theme_alt2_images) / sizeof(ImageTable);
 			load_image_table(&default_theme_alt2_images[0], n_default, NULL);
 			}
-		else if (GK.theme_alternative == 1)
+		else if (GK.theme_alternative == 1 || GK.theme_alternative == 4)
 			{
 			n_default = sizeof(default_theme_alt1_images) / sizeof(ImageTable);
 			load_image_table(&default_theme_alt1_images[0], n_default, NULL);
@@ -1197,7 +1312,7 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 	Monitor			*mon;
 	struct _config	*pc;
 	gchar			*s, *arg;
-	gint			i, val, foundit;
+	gint			i, val;
 	gchar			buf[CFG_BUFSIZE], pbuf[CFG_BUFSIZE];
 
 	strncpy(buf, line, CFG_BUFSIZE);	/* strtok() is destructive */
@@ -1269,7 +1384,8 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 	if (strcmp(arg, "on") == 0 || strcmp(arg, "true") == 0)
 		val = 1;
 
-	foundit = FALSE;
+	if (!cf)
+		return;
 	for (i = 0; i < size; ++i)
 		{
 		pc = cf + i;
@@ -1300,14 +1416,13 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 				}
 			if (pc->minimum > -100 && val < pc->minimum)
 				*(pc->value) = pc->minimum;
-			foundit = TRUE;
 			break;
 			}
 		}
 	}
 
 static void
-clear_style_list(GList *list, GList *name_list, gchar *debug_name)
+clear_style_list(GList *list, GList *name_list)
 	{
 	Style	*style;
 
@@ -1333,9 +1448,7 @@ static gchar	*base_theme[]	=
 	"StylePanel *.textcolor = white #000000 shadow",
 	"StylePanel *.alt_textcolor = #d8e0c8 #000000 shadow",
 	"StylePanel *.label_position = 50",
-	"StylePanel *.margin = 1",
-	"StylePanel *.top_margin = 2",
-	"StylePanel *.bottom_margin = 1",
+	"StylePanel *.margins = 1,1,2,1",
 	"StylePanel *.krell_yoff = 0",
 	"StylePanel *.krell_x_hot = 3",
 	"StylePanel *.krell_depth = 4",
@@ -1348,14 +1461,12 @@ static gchar	*base_theme[]	=
 	"StyleMeter *.textcolor = #ffeac4 #000000 shadow",
 	"StyleMeter *.alt_textcolor = wheat #000000 shadow",
 	"StyleMeter *.label_position = 50",
-	"StyleMeter *.margin = 2",
-	"StyleMeter *.top_margin = 2",
-	"StyleMeter *.bottom_margin = 2",
+	"StyleMeter *.margins = 2,2,2,2",
 	"StyleMeter *.krell_yoff = 1",
 	"StyleMeter *.krell_expand = 0",
-	"StyleMeter *.krell_ema_period = 1",
 	"StyleMeter *.krell_x_hot = -1",
 	"StyleMeter *.krell_depth = 1",
+	"StyleMeter *.krell_ema_period = 1",
 
 	/* These have an override effect */
 	"StyleMeter apm.bottom_margin = 2",
@@ -1421,9 +1532,11 @@ init_theme(void)
 	|  they are not, then they will be set to point to the default after
 	|  parsing the gkrellmrc file.
 	*/
-	clear_style_list(GK.chart_style_list, GK.chart_name_list, _("Chart"));
-	clear_style_list(GK.panel_style_list, GK.chart_name_list, _("Panel"));
-	clear_style_list(GK.meter_style_list, GK.meter_name_list, _("Meter"));
+	clear_style_list(GK.chart_style_list, GK.chart_name_list);
+	clear_style_list(GK.panel_style_list, GK.chart_name_list);
+	clear_style_list(GK.meter_style_list, GK.meter_name_list);
+	free_glist_and_data(&GK.custom_name_list);
+	free_glist_and_data(&GK.custom_style_list);
 
 	for (i = 0; i < sizeof(base_theme) / sizeof(gchar *); ++i)
 		parse_config_line(base_theme[i], &theme_config[0],
@@ -1500,9 +1613,7 @@ static gchar	*default_theme[]	=
 	"StylePanel *.alt_textcolor = #60fff0 #000000 shadow",
 
 	"StyleMeter *.border = 4,4,3,2",
-	"StyleMeter *.margin = 2",
-	"StyleMeter *.top_margin = 2",
-	"StyleMeter *.bottom_margin = 2",
+	"StyleMeter *.margins = 2,2,2,2",
 	"StyleMeter *.krell_yoff = 1",
 
 	"StyleMeter *.textcolor = #c8e4e2 #000000 shadow",
@@ -1517,7 +1628,6 @@ static gchar	*default_theme[]	=
 	"StyleMeter sensors.alt_textcolor = #c8e4e2 #000000 shadow",
 
 	"StyleMeter fs.border = 4,4,3,1",
-	"StyleMeter fs.top_margin = 2",
 	"StyleMeter fs.bottom_margin = 0",
 	"StyleMeter fs.label_position = 0",
 	"StyleMeter fs.alt_font = normal_font",
@@ -1535,7 +1645,9 @@ static gchar	*default_theme[]	=
 
 	"StyleMeter mem.krell_yoff = 0",
 	"StyleMeter mem.alt_font = normal_font",
-	"StyleMeter mem.bottom_margin = 1",
+	"StyleMeter mem.top_margin = 3",
+	"StyleMeter swap.top_margin = 3",
+	"StyleMeter swap.bottom_margin = 1",
 
 	"StyleMeter swap.krell_yoff = -2",		/* Bottom justify */
 	"StyleMeter swap.alt_font = normal_font",
@@ -1543,12 +1655,11 @@ static gchar	*default_theme[]	=
 	"StyleMeter sensors.alt_textcolor = #d8e0c8 #000000 shadow",
 	"StyleMeter sensors.top_margin = 3",
 	"StyleMeter sensors.bottom_margin = 3",
-	"set_image_border sensors_bg_volt 2,1,1,1",
+	"set_image_border sensors_bg_volt 1,1,1,1",
 
 	"StyleMeter timer.textcolor = #e8e4d2 #000000 shadow",
 	"StyleMeter timer.alt_textcolor = #c8e4e2 #000000 shadow",
-	"StyleMeter timer.margin = 1",
-	"StyleMeter timer.top_margin = 1",
+	"StyleMeter timer.margins = 1,1,1,2",
 	"set_image_border timer_bg_timer 1,1,2,2",
 
 	"StyleMeter uptime.textcolor = #e8e4d2 #000000 shadow",
@@ -1570,17 +1681,13 @@ static gchar	*default_theme[]	=
 	"set_integer gkrellmms_scroll_bottom_margin 2",
 	/* GKrellMMS time bar and button bar panels */
 	"StyleMeter gkrellmms.alt_textcolor = black #dcdccc shadow",
-	"StyleMeter gkrellmms.margin = 2",
-	"StyleMeter gkrellmms.top_margin = 2",
-	"StyleMeter gkrellmms.bottom_margin = 0",
+	"StyleMeter gkrellmms.margins = 2,2,2,0",
 	"StyleMeter gkrellmms.krell_yoff = 0",
 
 	/* Timers panels */
 	"spacer_top_height timers 3",
 	"spacer_bottom_height timers 3",
 	"StyleMeter timers.border = 6,6,2,2",
-	"StyleMeter timers.top_margin = 2",
-	"StyleMeter timers.bottom_margin = 2",
 	"StyleMeter timers.font = large_font",
 	"StyleMeter timers.textcolor = #d8e4d2 #000000 shadow",
 	"StyleMeter timers.alt_textcolor = #c8e4e2 #000000 shadow",
@@ -1651,7 +1758,7 @@ default_theme_config(void)
 	{
 	gint	i;
 
-	GK.theme_n_alternatives = 2;
+	GK.theme_n_alternatives = 5;
 	if (GK.theme_alternative > GK.theme_n_alternatives)
 		GK.theme_alternative = GK.theme_n_alternatives;
 
@@ -1659,18 +1766,20 @@ default_theme_config(void)
 		parse_config_line(default_theme[i], &theme_config[0],
 					sizeof(theme_config) / sizeof (struct _config));
 
-	if (GK.theme_alternative == 1)
+	if (GK.theme_alternative == 1 || GK.theme_alternative == 4)
 		{
 		for (i = 0; i < sizeof(default_theme_1) / sizeof(gchar *); ++i)
 			parse_config_line(default_theme_1[i], &theme_config[0],
 						sizeof(theme_config) / sizeof (struct _config));
 		}
-	if (GK.theme_alternative == 2)
+	if (GK.theme_alternative == 2 || GK.theme_alternative == 5)
 		{
 		for (i = 0; i < sizeof(default_theme_2) / sizeof(gchar *); ++i)
 			parse_config_line(default_theme_2[i], &theme_config[0],
 						sizeof(theme_config) / sizeof (struct _config));
 		}
+	if (GK.theme_alternative > 2)
+		parse_config_line("StyleChart *.transparency = 1", NULL, 0);
 	}
 
 
