@@ -20,6 +20,8 @@
 */
 
 /*
+|   2/8/2002  Christian Schroeder <chs@baltic-online.de> patch to avoid
+|               potential Solaris segfault in get_solaris_vfstab_list()
 | 10/29/2001  Darell Tan <dardil@singnet.com.sg> patch adds uid check for
 |               "owner" to fstab_user_permission().
 |  4/22/2001  Solaris code contributed by Daisuke Yabuki <dxy@acm.org>
@@ -109,7 +111,7 @@ static gchar	*eject_tray_command,
 /* ====== System dependent interface ====================================== */
 
 
-/* ----- FreeBSD & NetBSD ----------------------------------------------- */
+/* ----- FreeBSD NetBSD OpenBSD ----------------------------------------- */
 
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #include <sys/mount.h>
@@ -167,7 +169,7 @@ get_bsd_mounts_list()
 	}
 #endif
 
-#if defined(__FreeBSD__) || defined(__NetBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 static void
 eject_bsd_cdrom(gchar *device)
 	{
@@ -180,7 +182,7 @@ eject_bsd_cdrom(gchar *device)
 		close(d);
 		}
 	}
-#endif /* __FreeBSD__ || __NetBSD__ */
+#endif /* __FreeBSD__ || __NetBSD__ || __OpenBSD__ */
 
 
 #if defined(__FreeBSD__)
@@ -217,6 +219,14 @@ set_eject_command(void)
 	}
 #endif /* __NetBSD__ */
 
+#if defined(__OpenBSD__)
+static void
+set_eject_command(void)
+	{
+	return;
+	}
+#endif /* __OpenBSD__ */
+
 
 /* ----- Linux ------------------------------------------------------ */
 
@@ -307,7 +317,7 @@ eject_linux_cdrom(gchar *device)
 #if defined(CDROMEJECT)
 	gint	d;
 
-	if ((d = open(device, O_RDONLY)) >= 0)
+	if ((d = open(device, O_RDONLY|O_NONBLOCK)) >= 0)
 		{
 		ioctl(d, CDROMEJECT);
 		close(d);
@@ -321,7 +331,7 @@ set_eject_command(void)
 #if defined(WEXITSTATUS)
 	gint	n;
 
-	n = system("eject -n > /dev/null");
+	n = system("eject -d > /dev/null 2>&1");
 	if (WEXITSTATUS(n) == 0)
 		{
 		eject_tray_command = "eject %s";
@@ -456,7 +466,7 @@ get_solaris_vfstab_list(){
         return;
 
     while (getvfsent(fp, &vfsbuf) == 0) {
-        if (strcmp(vfsbuf.vfs_fstype, "ufs"))
+        if (!vfsbuf.vfs_fstype || strcmp(vfsbuf.vfs_fstype, "ufs"))
             continue;
         m = g_new0(Mount, 1);
         if(!vfsbuf.vfs_special) {
@@ -490,9 +500,9 @@ eject_solaris_cdrom(gchar *device) {
 #if defined(CDROMEJECT)
         gint    d;
 
-        if ((d = open(device, O_RDONLY)) >= 0) {
-                ioctl(d, CDROMEJECT);
-                close(d);
+		if ((d = open(device, O_RDONLY)) >= 0) {
+				ioctl(d, CDROMEJECT);
+				close(d);
         }
 #endif
 }
@@ -643,16 +653,12 @@ setup_fs_interface(void)
     {
 	uid = getuid();	/* only real root is allowed to mount/umount always */
 
-#if defined(__FreeBSD__) || defined(__NetBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 	get_mountlist = get_bsd_mounts_list;
 	get_fsusage = get_statfs_fsusage;
 	get_fstab_list = get_etc_fstab_list;
 	eject_cdrom = eject_bsd_cdrom;
 	set_eject_command();
-#elif defined(__OpenBSD__)
-	get_mountlist = get_bsd_mounts_list;
-	get_fsusage = get_statfs_fsusage;
-	get_fstab_list = get_etc_fstab_list;
 #elif defined(__linux__)
 	get_mountlist = get_linux_proc_mounts_list;
 	get_fsusage = get_statfs_fsusage;
@@ -1415,6 +1421,7 @@ static void
 create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 	{
 	Style		*style;
+	Margin		*m;
 	Panel		*p;
 
 	if (first_create)
@@ -1425,14 +1432,14 @@ create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 	fs->krell_factor = 1;
 
 	style = gkrellm_meter_style(style_id);
-
+	m = gkrellm_get_style_margins(style);
 	fs->text_decal = gkrellm_create_decal_text(p, "Afp0",
 			gkrellm_meter_textstyle(style_id), style, -1, -1, -1);
 
 	fs->mount_decal = gkrellm_create_decal_pixmap(p, GK.decal_misc_pixmap,
 			GK.decal_misc_mask, N_MISC_DECALS, style, -1, -1);
 	fs->mount_decal->x =
-				gkrellm_chart_width() - fs->mount_decal->w - style->margin;
+				gkrellm_chart_width() - fs->mount_decal->w - m->right;
 
 	if (fs->ejectable)
 		{
@@ -1995,7 +2002,8 @@ apply_fs_config(void)
 	gint			row;
 	GtkSpinButton	*spin;
 
-	cdrom_auto_eject = GTK_TOGGLE_BUTTON(cdrom_auto_eject_button)->active;
+	if (cdrom_auto_eject_button)
+		cdrom_auto_eject = GTK_TOGGLE_BUTTON(cdrom_auto_eject_button)->active;
 	binary_units = GTK_TOGGLE_BUTTON(binary_units_button)->active;
 	s = gkrellm_entry_get_text(&(GTK_COMBO(data_format_combo)->entry));
 	if (gkrellm_dup_string(&data_format, s))
@@ -2300,7 +2308,7 @@ N_("\tNotes: the mount point specified in a custom mount command\n"
    "\t\"Mount Point\" entry.  You should have the NOPASSWD option set\n"
    "\tin /etc/sudoers if using sudo.\n\n"),
 
-N_("<b>Primary and Secondary Monitors\n"),
+N_("<b>Primary and Secondary Monitors\n"), /* xgettext:no-c-format */
 N_("File system monitors can be created as primary (always visible)\n"
 	"or secondary which can be hidden and then shown when they are of\n"
 	"interest.  For example, you might make primary file system monitors\n"
