@@ -108,7 +108,7 @@
 */
 #define	GKRELLM_VERSION_MAJOR	1
 #define	GKRELLM_VERSION_MINOR	2
-#define	GKRELLM_VERSION_REV		8
+#define	GKRELLM_VERSION_REV		10
 /* Backward compatibility */
 #define	VERSION_MAJOR			GKRELLM_VERSION_MAJOR
 #define	VERSION_MINOR			GKRELLM_VERSION_MINOR
@@ -144,9 +144,13 @@
 
   /* Label midpoints are positioned as a percent of chart_width.
   */
-#define	LABEL_NONE		-1
-#define	LABEL_CENTER	50
-#define LABEL_MAX		100
+#define	GKRELLM_LABEL_NONE		-1
+#define	GKRELLM_LABEL_CENTER	50
+#define GKRELLM_LABEL_MAX		100
+  /* Pre 1.2.9 compatibility defines */
+#define	LABEL_NONE		GKRELLM_LABEL_NONE
+#define	LABEL_CENTER	GKRELLM_LABEL_CENTER
+#define LABEL_MAX		GKRELLM_LABEL_MAX
 
 #define	GRID_MODE_NORMAL		0
 #define	GRID_MODE_RESTRAINED	1
@@ -224,6 +228,7 @@ typedef struct
   */
 #define DF_LOCAL_PIXMAPS	0x1
 #define DF_TOP_LAYER		0x2
+#define	DF_MOVED			0x4
 
   /* Values for Decal state
   */
@@ -250,6 +255,7 @@ typedef struct
 	gint		x_off,
 				y_baseline;
 	TextStyle	text_style;	/* Used if decal is a drawn text string		*/
+	gint		x_old, y_old;	/* Used if decal was moved */
 	}
 	Decal;
 
@@ -296,22 +302,36 @@ typedef struct
 #define	SET_KRELL_X_HOT			3
 #define	SET_KRELL_EMA_PERIOD	4
 #define	SET_KRELL_DEPTH			5
-#define	SET_LABEL_POSITION		6
-#define	SET_MARGIN				7
-#define	SET_TOP_MARGIN			8
-#define	SET_BOTTOM_MARGIN		9
-#define	SET_TEXTCOLOR_A			10
-#define	SET_TEXTCOLOR_B			11
-#define	SET_TEXTFONT_A			12
-#define	SET_TEXTFONT_B			13
-#define	SET_BORDER				14
-#define	SET_TRANSPARENCY		15
+#define	SET_KRELL_LEFT_MARGIN	6
+#define	SET_KRELL_RIGHT_MARGIN	7
+#define	SET_LABEL_POSITION		8
+#define	SET_MARGIN				9
+#define	SET_MARGINS				10
+#define	SET_LEFT_MARGIN			11
+#define	SET_RIGHT_MARGIN		12
+#define	SET_TOP_MARGIN			13
+#define	SET_BOTTOM_MARGIN		14
+#define	SET_TEXTCOLOR_A			15
+#define	SET_TEXTCOLOR_B			16
+#define	SET_TEXTFONT_A			17
+#define	SET_TEXTFONT_B			18
+#define	SET_BORDER				19
+#define	SET_TRANSPARENCY		20
 
 #define	TS_A		1
 #define	TS_B		2
 
 #define BIT(a)	(1 << (a))
 
+typedef struct
+	{
+	gint	left,
+			right,
+			top,
+			bottom;
+	}
+	Margin;
+
   /* Some of these style entries do not apply to all monitors.
   */
 typedef struct
@@ -325,9 +345,8 @@ typedef struct
 				krell_ema_period,
 				krell_depth,
 				krell_spare;
-
 	gint		label_position,
-				margin,			/* Text left/right margins in a panel */
+				margin,			/* 1.2.9 deprecated, use margins */
 				spare;
 	TextStyle	label_tsA;
 	TextStyle	label_tsB;
@@ -335,9 +354,10 @@ typedef struct
 	GdkImlibBorder
 				border;			/* Border for background panel image */
 	gint		transparency;
-	gint		top_margin,		/* Text top margin in a panel */
-				bottom_margin;	/* Text bottom margin in a panel */
 	gboolean	themed;			/* True if theme has custom assignments */
+	Margin		margins;
+	gint		krell_left_margin,
+				krell_right_margin;
 	}
 	Style;
 
@@ -350,10 +370,13 @@ typedef struct
   |  wall full of power monitoring meters the Krell had in Forbidden Planet.
   */
   /* Krell.expand values */
-#define	KRELL_EXPAND_NONE		0
-#define	KRELL_EXPAND_LEFT		1
-#define	KRELL_EXPAND_RIGHT		2
-#define	KRELL_EXPAND_BAR_MODE	3
+#define	KRELL_EXPAND_NONE				0
+#define	KRELL_EXPAND_LEFT				1
+#define	KRELL_EXPAND_RIGHT				2
+#define	KRELL_EXPAND_BAR_MODE			3
+#define KRELL_EXPAND_LEFT_SCALED		4
+#define KRELL_EXPAND_RIGHT_SCALED		5
+#define	KRELL_EXPAND_BAR_MODE_SCALED	6
 
 typedef struct
 	{
@@ -386,7 +409,7 @@ typedef struct
 	gint		period;		/* of ema */
 	gint		full_scale_expand;
 	gboolean	monotonic;
-	gboolean	bar_mode;
+	gint		bar_mode;
 
 	Draw_rec	old_draw;	/* Last draw for restoring background.	*/
 	Draw_rec	draw;		/* Parameters to currently drawn krell	*/
@@ -922,6 +945,7 @@ struct	ThemeConfig
 
 	GList		*chart_name_list,		/* Move these lists to monitor_list */
 				*meter_name_list,
+				*custom_name_list,
 				*bg_chart_image_list,
 				*bg_grid_image_list,
 				*bg_panel_image_list,
@@ -930,7 +954,8 @@ struct	ThemeConfig
 				*krell_meter_image_list,
 				*chart_style_list,
 				*panel_style_list,
-				*meter_style_list;
+				*meter_style_list,
+				*custom_style_list;
 	};
 
   /* Debugs for debug_level	*/
@@ -1006,6 +1031,8 @@ typedef struct
 					created,
 					from_command_line,
 					spacers_shown;
+	Style			*panel_style,
+					*chart_style;
 	gchar			*style_name;
 	gint			style_type,
 					style_id;	/* helper until I consolidate image lists */
