@@ -31,6 +31,7 @@ Decal		*gkrellm_decal_new0(void);
 Label		*gkrellm_label_new0(void);
 Style		*gkrellm_style_new0(void);
 Style		*gkrellm_copy_style(Style *);
+void		gkrellm_copy_style_values(Style *, Style *);
 TextStyle	*gkrellm_textstyle_new0(void);
 TextStyle	*gkrellm_copy_textstyle(TextStyle *);
 
@@ -58,6 +59,7 @@ void		gkrellm_draw_chart_grid_line(Chart *, GdkPixmap *, gint);
 void		gkrellm_chart_bg_image_override(Chart *, GdkImlibImage *,
 						GdkImlibImage *);
 gint		gkrellm_chart_width(void);
+void		gkrellm_allow_scaling(gboolean *, gint *);
 void		gkrellm_set_chart_height_default(Chart *, gint);
 void		gkrellm_set_chart_height(Chart *, gint);
 gint		gkrellm_get_chart_scalemax(Chart *);
@@ -131,6 +133,8 @@ void		gkrellm_panel_bg_image_override(Panel *, GdkImlibImage *);
 /* ---- Krell functions ---- */
 Krell		*gkrellm_create_krell(Panel *, GdkImlibImage *, Style *);
 void		gkrellm_set_krell_full_scale(Krell *, gint, gint);
+void		gkrellm_set_style_krell_values(Style *, gint, gint, gint,
+						gint, gint, gint);
 void		gkrellm_set_krell_margins(Panel *, Krell *k, gint, gint);
 void		gkrellm_set_krell_expand(Style *, gchar *);
 void		gkrellm_update_krell(Panel *, Krell *, gulong);
@@ -226,6 +230,8 @@ void		gkrellm_config_message_window(gchar *title, gchar *message,
 						GtkWidget *);
 void		gkrellm_add_info_text(GtkWidget *, gchar **, gint);
 void		gkrellm_add_info_text_string(GtkWidget *, gchar *);
+Margin		*gkrellm_get_style_margins(Style *);
+void		gkrellm_set_style_margins(Style *, Margin *);
 void		gkrellm_get_top_bottom_margins(Style *, gint *, gint *);
 gboolean	gkrellm_style_is_themed(Style *);
 
@@ -256,8 +262,12 @@ gint		gkrellm_add_meter_style(Monitor *, gchar *);
 gint		gkrellm_lookup_chart_style_id(gchar *);
 gint		gkrellm_lookup_meter_style_id(gchar *);
 Style		*gkrellm_meter_style(gint);
-Style		*gkrellm_panel_style(gint n);
-Style		*gkrellm_chart_style(gint n);
+Style		*gkrellm_panel_style(gint);
+Style		*gkrellm_chart_style(gint);
+Style		*gkrellm_meter_style(gint);
+Style		*gkrellm_meter_style_by_name(gchar *);
+Style		*gkrellm_panel_style_by_name(gchar *);
+Style		*gkrellm_chart_style_by_name(gchar *);
 Style		*gkrellm_krell_slider_style(void);
 Style		*gkrellm_krell_mini_style(void);
 TextStyle 	*gkrellm_chart_textstyle(gint);
