@@ -75,7 +75,7 @@ message_window(gchar *title, gchar *message, GtkWidget *widget, gint mainwin)
 		widget = top_win;
 
 	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
-	gtk_window_set_title(GTK_WINDOW(window), title);
+	gtk_window_set_title(GTK_WINDOW(window), title ? title : "GKrellM");
 	gtk_container_border_width(GTK_CONTAINER(window), 8);
 
 	vbox = gtk_vbox_new(FALSE, 2);
