@@ -100,57 +100,52 @@ gkrellm_update_krell(Panel *p, Krell *k, gulong value)
 	|  frames in between at xnew proportional to ema/full_scale.
 	*/
 	d = k->depth;
-
+	if (k->ema == 0 || xnew == k->x0) /* Krell can settle before ema*/
+		frame = 0;
+	else if (k->ema >= k->full_scale)
+		frame = d - 1;
+	else
+		{
+		if (d == 1)
+			frame = 0;
+		else if (d == 2)
+			frame = 1;
+		else
+			frame = 1 + ((d - 2) * k->ema / k->full_scale);
+		}
+	y_src = k->h_frame * frame;
 	if (k->bar_mode)
 		{
-		frame = d - 1;
-		y_src = k->h_frame * frame;
-		x_src = k->x0;
+		x_src = (k->bar_mode == KRELL_EXPAND_BAR_MODE_SCALED) ? 0 : k->x0;
 		x_dst = k->x0;
 		w_overlap = xnew - k->x0;
 		}
 	else
 		{
-		if (k->ema == 0 || xnew == k->x0) /* Krell can settle before ema*/
-			frame = 0;
-		else if (k->ema >= k->full_scale)
-			frame = d - 1;
-		else
-			{
-			if (d == 1)
-				frame = 0;
-			else if (d == 2)
-				frame = 1;
-			else
-				frame = 1 + ((d - 2) * k->ema / k->full_scale);
-			}
-		y_src = k->h_frame * frame;
-
 		x_src = k->x_hot - (xnew - k->x0);
 		if (x_src < 0)
 			x_src = 0;
-
 		x_dst = xnew - k->x_hot;
 		if (x_dst < k->x0)
 			x_dst = k->x0;
-
 		w1 = k->w - x_src;
 		w2 = (k->x0 + k->w_scale + 1) - x_dst;
 		w_overlap = (w2 > w1) ? w1 : w2;
-
-		/* Clear the krell stencil bitmap and draw the mask.
-		*/
-		gdk_draw_rectangle(k->stencil, GK.bit0_GC, TRUE,
-					0,0,  k->w, k->h_frame);
-		if (k->mask)
-			gdk_draw_pixmap(k->stencil, GK.bit1_GC, k->mask,
-					x_src, y_src, x_dst, 0, w_overlap, k->h_frame);
-		else
-			gdk_draw_rectangle(k->stencil, GK.bit1_GC,
-					TRUE, x_dst, 0, w_overlap, k->h_frame);
 		}
 
-	k->old_draw = k->draw;
+	/* Clear the krell stencil bitmap and draw the mask.
+	*/
+	gdk_draw_rectangle(k->stencil, GK.bit0_GC, TRUE,
+				0,0,  k->w, k->h_frame);
+	if (k->mask)
+		gdk_draw_pixmap(k->stencil, GK.bit1_GC, k->mask,
+				x_src, y_src, x_dst, 0, w_overlap, k->h_frame);
+	else
+		gdk_draw_rectangle(k->stencil, GK.bit1_GC,
+				TRUE, x_dst, 0, w_overlap, k->h_frame);
+
+	if (!k->modified)		/* Preserve original old draw if there has been */
+		k->old_draw = k->draw;	/* no intervening draw_panel_layers() */
 
 	k->draw.x_src = x_src;
 	k->draw.y_src = y_src;
@@ -282,8 +277,14 @@ gkrellm_draw_panel_layers(Panel *p)
 		if (d->modified)
 			{
 			if (d->state != DS_INVISIBLE)
-				gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
+				{
+				if (d->flags & DF_MOVED)
+					gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
+						d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
+				else
+					gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
 						d->x, d->y,  d->x, d->y,   d->w, d->h);
+				}
 			p->modified = TRUE;
 			}
 		}
@@ -323,8 +324,16 @@ gkrellm_draw_panel_layers(Panel *p)
 			{
 			d = (Decal *) list->data;
 			if (d->state != DS_INVISIBLE && drawable)
-				gdk_draw_pixmap(drawable, GK.draw1_GC,
-						p->pixmap, d->x, d->y,  d->x, d->y,   d->w, d->h);
+				{
+				if (d->flags & DF_MOVED)
+					{
+					gdk_draw_pixmap(drawable, GK.draw1_GC, p->pixmap,
+						d->x_old, d->y_old, d->x_old, d->y_old, d->w, d->h);
+					d->flags &= ~DF_MOVED;
+					}
+				gdk_draw_pixmap(drawable, GK.draw1_GC, p->pixmap,
+						d->x, d->y,  d->x, d->y,   d->w, d->h);
+				}
 			d->modified = FALSE;
 			}
 		for (list = p->krell; list; list = list->next)
@@ -419,22 +428,11 @@ gkrellm_draw_decal_text(Panel *p, Decal *d, gchar *s, gint value)
 void
 gkrellm_move_krell_yoff(Panel *p, Krell *k, gint y)
 	{
-	Draw_rec	*dr, dr_old;
-
 	if (!k)
 		return;
-	dr_old = k->draw;
+	if (!k->modified)
+		k->old_draw = k->draw;
 	k->y0 = k->draw.y_dst = y;
-	dr = &dr_old;
-	if (p && p->pixmap && p->bg_pixmap)
-		{
-		gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
-				dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
-		if (p->drawing_area->window)
-			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
-					p->bg_pixmap,
-					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
-		}
 	k->modified = TRUE;
 	}
 
@@ -517,8 +515,6 @@ gkrellm_destroy_krell_list(Panel *p)
 void
 gkrellm_set_krell_margins(Panel *p, Krell *k, gint left, gint right)
 	{
-	Draw_rec	*dr;
-
 	if (!k)
 		return;
 	if (left > GK.chart_width - 3)
@@ -527,18 +523,8 @@ gkrellm_set_krell_margins(Panel *p, Krell *k, gint left, gint right)
 	k->w_scale  = GK.chart_width - k->x0 - right - 1;
 	if (k->w_scale < 2)
 		k->w_scale = 2;
-	k->modified = TRUE;
-
-	dr = &k->draw;
 	if (p && p->pixmap && p->bg_pixmap)
 		{
-		gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
-				dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
-		if (p->drawing_area->window)
-			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
-					p->bg_pixmap,
-					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
-
 		/* Move the krell proportionally between the new limits
 		*/
 		k->x_position = -1;
@@ -574,13 +560,21 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 		printf(_("create_krell: NULL image or style\n"));
 		exit(0);
 		}
-	k->x0 = 0;
+
+	/* Set left krell margin */
+	k->x0 = style->krell_left_margin;
+	if (k->x0 > GK.chart_width - 3)
+		k->x0 = GK.chart_width - 3;
+
+	/* Set right krell margin via w_scale (-1 adjust to keep x_hot visible */
+	k->w_scale  = GK.chart_width - k->x0 - style->krell_right_margin - 1;
+	if (k->w_scale < 2)
+		k->w_scale = 2;
+
 	k->y0 = style->krell_yoff;
 	if (k->y0 == -1)
 		gkrellm_get_top_bottom_margins(style, &k->y0, NULL);
 
-	k->w_scale  = GK.chart_width - k->x0 - 1;
-
 	w = im->rgb_width;
 	h = im->rgb_height;
 
@@ -588,7 +582,7 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 		style->krell_x_hot = w / 2;
 	w_render = w;
 	k->x_hot = style->krell_x_hot;
-	k->bar_mode = FALSE;
+	k->bar_mode = 0;
 
 	switch (style->krell_expand)
 		{
@@ -596,7 +590,11 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 			break;
 		case KRELL_EXPAND_BAR_MODE:
 			w_render = GK.chart_width;
-			k->bar_mode = TRUE;
+			k->bar_mode = KRELL_EXPAND_BAR_MODE;
+			break;
+		case KRELL_EXPAND_BAR_MODE_SCALED:
+			w_render = k->w_scale + 1;		/* undo -1 from w_scale calc */
+			k->bar_mode = KRELL_EXPAND_BAR_MODE_SCALED;
 			break;
 		case KRELL_EXPAND_LEFT:
 			if (style->krell_x_hot > 0 && style->krell_x_hot < w)
@@ -605,6 +603,13 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 				k->x_hot = GK.chart_width - 1;
 				}
 			break;
+		case KRELL_EXPAND_LEFT_SCALED:
+			if (style->krell_x_hot > 0 && style->krell_x_hot < w)
+				{
+				w_render = (k->w_scale + 1) * w / style->krell_x_hot;
+				k->x_hot = k->w_scale;
+				}
+			break;
 		case KRELL_EXPAND_RIGHT:
 			if (w > style->krell_x_hot)
 				{
@@ -612,6 +617,13 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 				k->x_hot = style->krell_x_hot * w_render / w;
 				}
 			break;
+		case KRELL_EXPAND_RIGHT_SCALED:
+			if (w > style->krell_x_hot)
+				{
+				w_render = (k->w_scale + 1) * w / (w - style->krell_x_hot);
+				k->x_hot = style->krell_x_hot * w_render / w;
+				}
+			break;
 		}
 
 	gdk_imlib_render(im, w_render, h);
@@ -633,18 +645,6 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 		}
 
 	k->stencil = gdk_pixmap_new(top_win->window, GK.chart_width, k->h_frame,1);
-	if (k->bar_mode)
-		{	/* Stencil is never modified for bar mode, so draw it once */
-		gdk_draw_rectangle(k->stencil, GK.bit0_GC, TRUE,
-					0, 0,  k->w, k->h_frame);
-		if (k->mask)
-			gdk_draw_pixmap(k->stencil, GK.bit1_GC, k->mask,
-					0, k->h_frame * (k->depth - 1), 0, 0, k->w, k->h_frame);
-		else
-			gdk_draw_rectangle(k->stencil, GK.bit1_GC, TRUE,
-					0, 0, k->w, k->h_frame);
-		}
-
 	k->period = style->krell_ema_period;
 	if (k->period >= 4 * GK.update_HZ)
 		k->period = 4 * GK.update_HZ;
@@ -678,22 +678,16 @@ disconnect_panel_signals(Panel *p, DecalButton *b)
 void
 gkrellm_move_decal(Panel *p, Decal *d, gint x, gint y)
 	{
-	gint	x_old, y_old;
-
-	if (!d)
+	if (!d || (d->x == x && d->y == y))
 		return;
-	x_old = d->x;
-	y_old = d->y;
+	if (!(d->flags & DF_MOVED))	/* If no intervening draw_panel_layers() */
+		{						/* then preserve original old position   */
+		d->x_old = d->x;
+		d->y_old = d->y;
+		}
 	d->x = x;
 	d->y = y;
-	if (p && p->pixmap && p->bg_pixmap)
-		{
-		gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
-					x_old, y_old,  x_old, y_old,   d->w, d->h);
-		if (p->drawing_area->window)
-			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
-					p->bg_pixmap, x_old, y_old,  x_old, y_old,   d->w, d->h);
-		}
+	d->flags |= DF_MOVED;
 	d->modified = TRUE;
 	}
 
@@ -797,6 +791,7 @@ gkrellm_create_decal_pixmap(Panel *p, GdkPixmap *pixmap, GdkBitmap *mask,
 			gint depth, Style *style, gint x, gint y)
 	{
 	GtkWidget	*top_win;
+	Margin		*m;
 	Decal		*d;
 
 	if (!pixmap)
@@ -812,16 +807,17 @@ gkrellm_create_decal_pixmap(Panel *p, GdkPixmap *pixmap, GdkBitmap *mask,
 		d->h = 1;
 
 	d->x = x;
+	m = gkrellm_get_style_margins(style);
 	if (d->x < 0 && style)
 		{
-		if (style->label_position < 50)
-			d->x = GK.chart_width - d->w - style->margin;
+		if (style->label_position < 50 && style->label_position != LABEL_NONE)
+			d->x = GK.chart_width - d->w - m->right;
 		else
-			d->x = style->margin;
+			d->x = m->left;
 		}
 	d->y = y;
 	if (d->y < 0)
-		gkrellm_get_top_bottom_margins(style, &d->y, NULL);
+		d->y = m->top;
 
 	d->pixmap = pixmap;
 	d->mask   = mask;
@@ -838,6 +834,7 @@ gkrellm_create_decal_text(Panel *p, gchar *string, TextStyle *ts,
 			Style *style, gint x, gint y, gint w)
 	{
 	GtkWidget	*top_win;
+	Margin		*m;
 	Decal		*d;
 	gint		ll, rr, ww, aa, dd;
 
@@ -860,18 +857,19 @@ gkrellm_create_decal_text(Panel *p, gchar *string, TextStyle *ts,
 	d->h = aa + dd + d->text_style.effect;
 	if (style)
 		{
+		m = gkrellm_get_style_margins(style);
 		if (w < 0)
-			d->w = GK.chart_width - 2 * style->margin;
+			d->w = GK.chart_width - m->left - m->right;
 		else if (w == 0)
 			d->w = ww + d->text_style.effect;
 		else
 			d->w = w;
 
-		d->x = (x >= 0) ? x : style->margin;
+		d->x = (x >= 0) ? x : m->left;
 
 		d->y = y;
 		if (d->y < 0)
-			gkrellm_get_top_bottom_margins(style, &d->y, NULL);
+			d->y = m->top;
 		}
 	else
 		{
@@ -1218,12 +1216,10 @@ gkrellm_make_overlay_button(Panel *p, void (*func)(), void *data,
 	Decal		*d;
 	GdkPixmap	*pm, *pixmap;
 	GdkBitmap	*m, *mask;
-	gint		margin;
 
 	if (!p)
 		return NULL;
 	top_win = gkrellm_get_top_window();
-	margin = (p->style) ? p->style->margin : 0;
 	if (x < 0)
 		{
 		w -= x;
@@ -1354,6 +1350,7 @@ put_label_in_button(Panel *p, void (*func)(), void *data, gint type, gint pad)
 	{
 	DecalButton		*b;
 	Label			*lbl;
+	Margin			*m;
 	GdkImlibImage	*normal_image, *pressed_image;
 	GdkImlibBorder	*style_border, *fr_border;
 	gint			x, y, w, h;
@@ -1371,10 +1368,11 @@ put_label_in_button(Panel *p, void (*func)(), void *data, gint type, gint pad)
 	*/
 	if ((lbl = p->label) == NULL || lbl->string == NULL || lbl->position < 0)
 		{
+		m = gkrellm_get_style_margins(p->style);
 		style_border = p->style ? &p->style->border : &zero_border;
-		x = 0 + (p->style) ? p->style->margin : 0;
+		x = 0 + m->left;
 		y = 0 + style_border->top - fr_border->top;
-		w = p->w - x -  ((p->style) ? p->style->margin : 0);
+		w = p->w - x - m->right;
 		h = p->h - y - style_border->bottom + fr_border->bottom;
 		}
 	else
