@@ -20,8 +20,10 @@
 */
 
 /*
-|  3/22/2000 Patch from Jason Kasper <j_dot_kasper@usa.net>:
-|		Added option to run in withdrawn mode for the Blackbox slit.
+|  1/14/2002 Patch from Dj-Death <djdeath@gmx.fr>
+|	Added updated_background() so transparency will track background changes.
+|  3/22/2000 Patch from Jason Kasper <j_dot_kasper@usa.net>
+|	Added option to run in withdrawn mode for the Blackbox slit.
 */
 
 #include "gkrellm.h"
@@ -37,7 +39,7 @@
 kvm_t	*kvmd = NULL;
 char	errbuf[_POSIX2_LINE_MAX];
 #endif
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) && defined(__i386__)
 extern void scan_for_sensors();
 #endif
 
@@ -112,7 +114,6 @@ gint			w_display, h_display;
 gint			window_decorations;
 time_t			time_now;
 
-
 static GtkItemFactory *item_factory;
 
 static Pixmap	root_xpixmap	= None;
@@ -397,6 +398,42 @@ set_or_save_position(gint save)
 		}
 	}
 
+  /* Check if background has changed */
+static gboolean
+updated_background(void)
+	{
+	Pixmap	root_pix = None;
+	Atom	prop, ret_type = (Atom) 0;
+	guchar	*prop_return = NULL;
+	gint	fmt;
+	gulong	nitems, bytes_after;
+  
+	if (no_transparency || !GK.any_transparency)
+		return FALSE;
+	prop = XInternAtom(GDK_DISPLAY(), "_XROOTPMAP_ID", True);
+	if (prop == None)
+		return FALSE;
+  
+	XGetWindowProperty(GDK_DISPLAY(), GDK_ROOT_WINDOW(), prop, 0L, 1L, False,
+			AnyPropertyType, &ret_type, &fmt, &nitems, &bytes_after,
+			&prop_return);
+	if (prop_return && ret_type == XA_PIXMAP)
+		{
+		root_pix = *((Pixmap *) prop_return);
+		XFree(prop_return);
+		}
+	else
+		return FALSE;
+
+	if (root_pix != root_xpixmap)
+		{
+		root_xpixmap = root_pix;
+		return TRUE;
+		}
+	return FALSE;
+	}
+
+
 static gint
 update_monitors()
 	{
@@ -448,6 +485,12 @@ update_monitors()
 	if (save_position_countdown > 0 && --save_position_countdown == 0)
 		set_or_save_position(1);
 
+	/* Update if background has changed */
+	if (   GK.second_tick && updated_background()
+		&& check_rootpixmap_transparency == 0
+	   )
+		check_rootpixmap_transparency = 1;
+
 	if (check_rootpixmap_transparency > 0)
 		if (--check_rootpixmap_transparency == 0)
 			apply_rootpixmap_transparency();
@@ -512,8 +555,8 @@ gkrellm_get_top_bottom_margins(Style *style, gint *top, gint *bottom)
 		{
 		if (GK.use_top_bottom_margins)
 			{
-			t = style->top_margin;
-			b = style->bottom_margin;
+			t = style->margins.top;
+			b = style->margins.bottom;
 
 			/* Ugh!! If a plugin violates policy and declares a static
 			|  Style struct (and it's so), then pre 1.0.3 it won't have
@@ -537,6 +580,25 @@ gkrellm_get_top_bottom_margins(Style *style, gint *top, gint *bottom)
 		*bottom = b;
 	}
 
+Margin *
+gkrellm_get_style_margins(Style *style)
+	{
+	static Margin	m_default = {0,0,0,0};
+
+	if (!style)
+		return &m_default;
+	gkrellm_get_top_bottom_margins(style,
+				&style->margins.top, &style->margins.bottom);
+	return &style->margins;
+	}
+
+void
+gkrellm_set_style_margins(Style *style, Margin *margins)
+	{
+	if (style && margins)
+		style->margins = *margins;
+	}
+
 void
 gkrellm_draw_string(GdkDrawable *drawable, TextStyle *ts, gint x, gint y,
 			gchar *s)
@@ -1737,17 +1799,17 @@ load_builtin_monitors()
 	gkrellm_add_chart_style(NULL, "*");
 	gkrellm_add_meter_style(NULL, "*");
 
-	/* The sensors_monitor does not have a create or update, but it does
+	/* The sensors config does not have a create or update, but it does
 	|  have an apply which needs to be called before the cpu or proc apply.
-	|  So just put sensor first.
+	|  So just put sensors config first.
 	*/
-	add_builtin(init_sensors_monitor());
+	add_builtin(init_sensors_config_monitor());
 	add_builtin(init_host_monitor());
 	add_builtin(init_cal_monitor());
 	add_builtin(init_clock_monitor());
 	add_builtin(init_cpu_monitor());
 	add_builtin(init_proc_monitor());
-	add_builtin(init_voltage_monitor());
+	add_builtin(init_sensor_monitor());
 	add_builtin(init_disk_monitor());
 	add_builtin(init_inet_monitor());
 	add_builtin(init_net_monitor());
@@ -1804,7 +1866,9 @@ main (int argc, char *argv[])
 		}
 #endif
 #if defined(__FreeBSD__)
+#if defined(__i386__)
 	scan_for_sensors();
+#endif
 	if (setuid(getuid()) != 0)
 		{
 		fprintf(stderr, "Can't drop setuid privileges.");
@@ -1872,9 +1936,12 @@ main (int argc, char *argv[])
 #endif
 
 	/* Even when I destroy images, if same im pointer turns up in image
-	| reload, pixmap cacheing can be fooled. So turn off pixmap cacheing.
+	| reload, pixmap caching can be fooled. So turn off pixmap caching.
+	| Also, if images in two monitor subdirs happen to be the same, image
+	| caching can prevent different borders from being applied.  So turn
+	| off image caching.
 	*/
-	gdk_imlib_set_cache_info(0, 1);		/* cache_pixmaps, cache_images	*/
+	gdk_imlib_set_cache_info(0, 0);		/* cache_pixmaps, cache_images	*/
 
 	GK.start_time = time(0);
 	gkrellm_current_tm = *(localtime(&GK.start_time));
@@ -2031,7 +2098,6 @@ main (int argc, char *argv[])
 		} 
 	if (on_top)
 		set_on_top();
-
 	start_timer(GK.update_HZ);
 	setup_signal_handler();
 	gtk_main ();
