@@ -1474,6 +1474,11 @@ create_mem_panel(GtkWidget *vbox, gint first_create)
 				gkrellm_meter_textstyle(mm->style_id), s, -1, -1, -1);
 	gkrellm_panel_configure(p, NULL, s);
 	gkrellm_panel_create(vbox, mon_mem, p);
+
+	if (s->label_position == GKRELLM_LABEL_NONE)
+		mem.label = "";
+	else
+		mem.label = _("Mem");
 	ts = &mm->decal_label->text_style;
 	mm->x_label = x_label_position(p->label->position, mm->decal_label->w,
 				gdk_string_width(ts->font, mm->label), 0);
@@ -1497,7 +1502,7 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 	{
 	MeminfoMeter	*mm;
 	Panel			*p;
-	Style			*style;
+	Style			*style, *panel_style;
 	TextStyle		*ts;
 
 	mm = &swap;
@@ -1505,13 +1510,18 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 		mm->panel = gkrellm_panel_new0();
 	p = mm->panel;
 
-	/* Need a chart styled krell on the swap meter panel
-	*/
-	mm->krell_delta = gkrellm_create_krell(p,
-			gkrellm_krell_panel_image(DEFAULT_STYLE_ID),
-			gkrellm_panel_style(DEFAULT_STYLE_ID));
-
 	style = gkrellm_meter_style(mm->style_id);
+
+	/* Need a chart styled krell on the swap meter panel, but want it to track
+	|  the meter krell margins.
+	*/
+	panel_style = gkrellm_copy_style(gkrellm_panel_style(DEFAULT_STYLE_ID));
+	panel_style->krell_left_margin = style->krell_left_margin;
+	panel_style->krell_right_margin = style->krell_right_margin;
+	mm->krell_delta = gkrellm_create_krell(p,
+			gkrellm_krell_panel_image(DEFAULT_STYLE_ID), panel_style);
+	g_free(panel_style);	/* unlike panels, krell styles need not persist */
+
 	mm->krell_used = gkrellm_create_krell(p,
 				gkrellm_krell_meter_image(mm->style_id), style);
 	gkrellm_monotonic_krell_values(mm->krell_used, FALSE);
@@ -1522,6 +1532,10 @@ create_swap_panel(GtkWidget *vbox, gint first_create)
 	gkrellm_panel_configure(p, NULL, style);
 	gkrellm_panel_create(vbox, mon_swap, p);
 
+	if (style->label_position == GKRELLM_LABEL_NONE)
+		swap.label = "";
+	else
+		swap.label = _("Swap");
 	ts = &mm->decal_label->text_style;
 	mm->x_label = x_label_position(p->label->position, mm->decal_label->w,
 				gdk_string_width(ts->font, mm->label), 0);
@@ -1545,9 +1559,6 @@ create_mem(GtkWidget *vbox, gint first_create)
 	{
 	if (first_create)
 		{
-		mem.label = _("Mem");
-		swap.label = _("Swap");
-
 		swap_chart.vbox = gtk_vbox_new(FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(vbox), swap_chart.vbox, FALSE, FALSE, 0);
 
