@@ -298,6 +298,9 @@ sync_bsd_net_interfaces()
 /* See linux/net/ipv4/route.c (or ipv6) for /proc/net/route code */
 /* See linux/net/core/dev.c for /proc/net/dev code */
 
+#include <limits.h>
+#include <errno.h>
+
 #define	PROC_NET_DEV_FILE	"/proc/net/dev"
 #define	PROC_NET_ROUTE_FILE	"/proc/net/route"
 #define	LOCK_DIRECTORY		"/var/lock"
@@ -328,6 +331,9 @@ read_linux_net_data(void)
 	gint	i;
 	gulong	rx_packets	= 0,
 			tx_packets	= 0;
+#if defined(G_HAVE_GINT64)
+	guint64	ll;
+#endif
 
 	if ((f = fopen(PROC_NET_DEV_FILE, "r")) == NULL)
 		return;
@@ -365,9 +371,28 @@ read_linux_net_data(void)
 				if ((s = strtok(NULL, " \t\n")) == NULL)
 					break;
 				if (i == rx_bytes_index)
+					{
 					net->rx = strtoul(s, NULL, 0);
+#if defined(G_HAVE_GINT64)
+					/* Can have 32 bit library / 64 bit kernel mismatch.
+					|  If so, just using the 32 low bits of a long long is OK.
+					*/
+					if (   net->rx == ULONG_MAX && errno == ERANGE
+						&& sscanf(s, "%Ld", &ll) == 1
+					   )
+						net->rx = (gulong) ll;
+#endif
+					}
 				else if (i == tx_bytes_index)
+					{
 					net->tx = strtoul(s, NULL, 0);
+#if defined(G_HAVE_GINT64)
+					if (   net->tx == ULONG_MAX && errno == ERANGE
+						&& sscanf(s, "%Ld", &ll) == 1
+					   )
+						net->tx = (gulong) ll;
+#endif
+					}
 				else if (i == rx_packets_index)
 					rx_packets = strtoul(s, NULL, 0);
 				else if (i == tx_packets_index)
@@ -410,7 +435,6 @@ sync_linux_net_interfaces(void)
 		while (fgets(buf, sizeof(buf), f))
 			{
 			if (   ((s = strtok(buf, " \t\n")) == NULL)
-//				|| strncmp(s, "lo", 2) == 0
 				|| strncmp(s, "dummy", 5) == 0
 				|| (*s == '*' && *(s+1) == '\0')
 			   )
@@ -1787,6 +1811,7 @@ create_net_timer(GtkWidget *vbox, gint first_create)
 	{
 	Panel			*p;
 	Style			*style;
+	Margin			*m;
 	GdkImlibBorder	*tb;
 	gint			top_margin, bot_margin;
 	gint			x, y, w, w1, h, w_avail;
@@ -1796,11 +1821,12 @@ create_net_timer(GtkWidget *vbox, gint first_create)
 	p = timer_panel;
 
 	style = gkrellm_meter_style(timer_style_id);
+	m = gkrellm_get_style_margins(style);
 	tb = &bg_timer_style->border;
 
 	button_decal = gkrellm_create_decal_pixmap(p, decal_timer_button_pixmap,
 			decal_timer_button_mask, N_TB_DECALS, style, -1, -1);
-	button_decal->x = gkrellm_chart_width() - style->margin - button_decal->w;
+	button_decal->x = gkrellm_chart_width() - m->right - button_decal->w;
 
 	if (timer_seconds)
 		seconds_decal = gkrellm_create_decal_text(p, "00",
@@ -1813,7 +1839,7 @@ create_net_timer(GtkWidget *vbox, gint first_create)
 	gkrellm_panel_configure(p, NULL, style);
 
 	w = time_decal->w + (seconds_decal ? (seconds_decal->w + SEC_PAD) : 0);
-	w_avail = button_decal->x - style->margin;
+	w_avail = button_decal->x - m->left;
 	if (bg_timer_image)
 		w_avail -= tb->left + tb->right;
 	wide_timer = (w > w_avail || ! timer_seconds) ? FALSE : TRUE;
