@@ -78,12 +78,23 @@ gkrellm_style_new0(void)
 Style *
 gkrellm_copy_style(Style *style)
 	{
-	Style *s	= gkrellm_style_new0();
+	Style *s = NULL;
 
-	*s = *style;
+	if (style)
+		{
+		s = gkrellm_style_new0();
+		*s = *style;
+		}
 	return s;
 	}
 
+void
+gkrellm_copy_style_values(Style *src, Style *dst)
+	{
+	if (src && dst)
+		*src = *dst;
+	}
+
 gboolean
 gkrellm_style_is_themed(Style *style)
 	{
@@ -173,6 +184,51 @@ gkrellm_chart_style(gint n)
 	return get_style_from_list(GK.chart_style_list, n);
 	}
 
+static Style *
+get_style_from_list_by_name(GList *name_list, GList *style_list, gchar *name)
+	{
+	GList	*list;
+	gchar	*p, buf[128];
+	gint	n;
+
+	strncpy(buf, name, sizeof(buf));
+	buf[sizeof(buf) - 1] = '\0';
+	if (   (p = strchr(buf, '.')) != NULL
+		&& (n = string_position_in_list(GK.custom_name_list, name)) >= 0
+	   )
+		list = GK.custom_style_list;
+	else
+		{
+		if (p)
+			*p = '\0';
+		if ((n = string_position_in_list(name_list, buf)) < 0)
+			n = 0;
+		list = style_list;
+		}
+	return get_style_from_list(list, n);
+	}
+
+Style *
+gkrellm_meter_style_by_name(gchar *name)
+	{
+	return get_style_from_list_by_name(GK.meter_name_list,
+				GK.meter_style_list, name);
+	}
+
+Style *
+gkrellm_panel_style_by_name(gchar *name)
+	{
+	return get_style_from_list_by_name(GK.chart_name_list,
+				GK.panel_style_list, name);
+	}
+
+Style *
+gkrellm_chart_style_by_name(gchar *name)
+	{
+	return get_style_from_list_by_name(GK.chart_name_list,
+				GK.chart_style_list, name);
+	}
+
 gint
 gkrellm_lookup_chart_style_id(gchar *name)
 	{
@@ -180,7 +236,7 @@ gkrellm_lookup_chart_style_id(gchar *name)
 	gint	i;
 
 	for (list = GK.chart_name_list, i = 0; list; list = list->next, ++i)
-		if (!strcmp(name, (gchar *)list->data))
+		if (name && !strcmp(name, (gchar *)list->data))
 			return i;
 	return 0;
 	}
@@ -192,7 +248,7 @@ gkrellm_lookup_meter_style_id(gchar *name)
 	gint	i;
 
 	for (list = GK.meter_name_list, i = 0; list; list = list->next, ++i)
-		if (!strcmp(name, (gchar *)list->data))
+		if (name && !strcmp(name, (gchar *)list->data))
 			return i;
 	return 0;
 	}
@@ -501,6 +557,14 @@ gkrellm_chart_width(void)
 	return GK.chart_width;
 	}
 
+void
+gkrellm_allow_scaling(gboolean *allow, gint *width_ref)
+	{
+	if (allow)
+		*allow = GK.allow_scaling;
+	if (width_ref)
+		*width_ref = GK.chart_width_ref;
+	}
 
 /* ======================================================================= */
 
