@@ -19,16 +19,6 @@
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* This file contains the general sensor interface as well as a voltage
-|  monitor.  The general sensor interface provides data and configuration
-|  for temperature, fan, and voltage sensors.  The monitoring of this data
-|  is handled here (voltage monitor) and in the CPU and Proc monitors (fan
-|  and temperature monitoring).  So the organization here is:
-|       sensor interface (temperature, fan, voltage)
-|       voltage monitor
-|       unified config for sensor interface and voltage monitor
-*/
-
 /*
 |  10/29/2001  NetBSD code contributed by
 |                Anthony Mallet anthony.mallet@useless-ficus.net
@@ -228,7 +218,7 @@ static VoltDefault	voltdefault8[] =
 /* ====== System dependent interface ====================================== */
 
 /* ------- FreeBSD ------------------------------------------------------- */
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) && defined(__i386__)
 #include <osreldate.h>
 #include <machine/cpufunc.h>
 #if __FreeBSD_version >= 300000
@@ -488,7 +478,7 @@ get_linux_sensor(Sensor *sensor)
 		}
 	else if (sensor->type == SENSOR_FAN)
 		{
-		if ((n = sscanf(buf, "%f %f", &t[0], &t[1])) > 0)
+		if ((n = sscanf(buf, "%f %f %f", &t[0], &t[1], &t[2])) > 0)
 			sensor->raw_value = t[n - 1];
 		else
 			return FALSE;
@@ -805,7 +795,8 @@ get_netbsd_sensor(Sensor *sensor)
 
 
 /* ----- Other Systems -------------------------------------------------- */
-#if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__)
+#if !(defined(__FreeBSD__) && defined(__i386__)) && \
+    !defined(__linux__) && !defined(__NetBSD__)
 
 #define	SENSORS_DIR		""
 
@@ -817,7 +808,7 @@ get_netbsd_sensor(Sensor *sensor)
 static gboolean
 setup_sensor_interface(void)
 	{
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) && defined(__i386__)
 	get_temperature = get_freebsd_sensor;
 	get_fan = get_freebsd_sensor;
 	get_voltage = get_freebsd_sensor;
@@ -842,6 +833,7 @@ setup_sensor_interface(void)
 
 /* ======================================================================== */
 static gboolean	use_threads,
+				thread_data_valid,
 				units_fahrenheit;
 
 #if defined(GKRELLM_THREADS)
@@ -959,6 +951,7 @@ void
 sensor_create_decals(Panel *p, gint style_id, Decal **dsensor, Decal **dfan)
 	{
 	Style		*style;
+	Margin		*m;
 	TextStyle	*ts;
 	Decal		*ds	= NULL,
 				*df	= NULL;
@@ -967,9 +960,10 @@ sensor_create_decals(Panel *p, gint style_id, Decal **dsensor, Decal **dfan)
 	if (sensor_list || GK.demo)
 		{
 		style = gkrellm_panel_style(style_id);
+		m = gkrellm_get_style_margins(style);
 		ts = gkrellm_panel_alt_textstyle(style_id);
-		margin = style->margin;
-		w_avail = gkrellm_chart_width() - 2 * margin;
+		margin = m->left;
+		w_avail = gkrellm_chart_width() - m->left - m->right;
 
 		/* Fan decal goes at left margin
 		*/
@@ -978,15 +972,15 @@ sensor_create_decals(Panel *p, gint style_id, Decal **dsensor, Decal **dfan)
 		/* Sensor decal (fan and/or temp) carves out space remaining to right.
 		|  Try to get enough for .1 deg resolution, otherwise what is left.
 		*/
-		w = gdk_string_width(ts->font, "188.8C");
-		if (w >= w_avail - df->w)
+		w = gdk_string_width(ts->font, "188.8F");
+		if (w > w_avail - df->w)
 			w = gdk_string_width(ts->font, "88.8C");
 		if (w >= w_avail - df->w - 8)
 			{
 			/* violate margins to try a squeeze fit
 			*/
 			margin = 0;
-			w_avail = gkrellm_chart_width() - 2 * margin;
+			w_avail = gkrellm_chart_width();
 			if (w > w_avail - df->w - 4)
 				w = w_avail - df->w - 4;	/* units pushed off the right */
 			}
@@ -1027,7 +1021,7 @@ gkrellm_sensor_draw_temperature_decal(Panel *p, Decal *d, gfloat t,
 	if ((s = strchr(buf, '.')) == NULL)
 		s = strchr(buf, ',');			/* Locale may use commas */
 	w = gdk_string_measure(d->text_style.font, buf) + d->text_style.effect;
-	if (w > d->w)
+	if (w > d->w + 1)
 		{
 		snprintf(buf, sizeof(buf), "%.0f%c", t, units);
 		w = gdk_string_measure(d->text_style.font, buf) + d->text_style.effect;
@@ -1145,7 +1139,7 @@ gkrellm_sensor_read_temperature(gchar *name, gfloat *temp, gchar *units)
 		&& get_temperature
 	   )
 		{
-		found_temp = use_threads ? TRUE : (*get_temperature)(sensor);
+		found_temp = thread_data_valid ? TRUE : (*get_temperature)(sensor);
 		sensor->value = sensor->raw_value * sensor->factor + sensor->offset;
 		if (units_fahrenheit)
 			sensor->value = 1.8 * sensor->value + 32.0;
@@ -1181,7 +1175,7 @@ gkrellm_sensor_read_fan(gchar *name, gfloat *fan)
 		&& get_fan
 	   )
 		{
-		found_fan = use_threads ? TRUE : (*get_fan)(sensor);
+		found_fan = thread_data_valid ? TRUE : (*get_fan)(sensor);
 		sensor->value = sensor->raw_value * sensor->factor;
 		f = sensor->value;
 		}
@@ -1208,7 +1202,7 @@ sensor_read_voltage(Sensor *sensor, gfloat *voltage)
 
 	if (sensor && get_voltage)
 		{
-		found_voltage = use_threads ? TRUE : (*get_voltage)(sensor);
+		found_voltage = thread_data_valid ? TRUE : (*get_voltage)(sensor);
 		offset = sensor->offset;
 		if (sensor->vref)	/* A negative voltage is level shifted by vref */
 			offset *= sensor->vref->value;
@@ -1240,29 +1234,34 @@ gkrellm_sensor_read_voltage(gchar *name, gfloat *voltage)
 	}
 
 /* =================================================================== */
-/* The voltage monitor */
+/* The sensors monitor */
 
 #define	SENSOR_STYLE_NAME	"sensors"
 
 
 typedef struct
 	{
+	Panel	**panel;
 	Decal	*name_decal,
-			*volt_decal;
+			*sensor_decal;
 	Sensor	*sensor;
 	}
-	VoltMon;
+	SensorMon;
 
-GList	*volt_list;
+GList	*volt_list,
+		*temperature_list,
+		*fan_list;
 
-GtkWidget		*voltage_vbox;
+GtkWidget		*sensors_vbox;
 
-static Panel	*pVolt;
+static Panel	*pVolt,
+				*pTemp,
+				*pFan;
 static gint		style_id;
 static gint		volt_mon_width,
 				volt_mon_height,
 				volt_name_width,
-				bezel_width;
+				volt_bezel_width;
 
   /* Display modes */
 #define	DIGITAL_WITH_LABELS	0
@@ -1272,15 +1271,15 @@ static gint		volt_mon_width,
 #define	MONITOR_PAD		6
 #define	NAME_PAD		4
 
-Monitor			*mon_volt;
 Monitor			*mon_sensors;
+Monitor			*mon_config_sensors;
 
 static GdkImlibImage	*bezel_image;
 
 static Style	*bezel_style;		/* Just for the bezel image border */
 
 static gint		display_mode,
-				voltages_modified,
+				sensor_monitor_modified,
 				correction_modified,
 				have_negative_volts;
 
@@ -1291,10 +1290,32 @@ static gint		pixel_grub;
 
 static Sensor	*demo0_sensor, *demo1_sensor;
 
-static void
-add_volt_mon(Sensor *s)
+  /* The price of having magic reserved names for the Proc and CPU monitors.
+  */
+static gboolean
+can_monitor_sensor_name(Sensor *s)
 	{
-	VoltMon		*volt;
+	gchar	d, *name = s->name;
+
+	if (!*name)
+		return FALSE;
+	if (!strcmp(name, "mb"))
+		return FALSE;
+	if (!strncmp(name, "cpu", 3))
+		{
+		if (strlen(name) == 3 && n_smp_cpus == 0)
+			return FALSE;
+		d = name[3] - '0';
+		if (strlen(name) == 4 && n_smp_cpus > 0 && d >= 0 && d < n_smp_cpus)
+			return FALSE;
+		}
+	return TRUE;
+	}
+
+static void
+add_sensor_monitor(Sensor *s)
+	{
+	SensorMon		*smon;
 	VoltDefault	*vdf;
 
 	if (s->type == SENSOR_VOLTAGE)
@@ -1303,11 +1324,27 @@ add_volt_mon(Sensor *s)
 		if (!s->has_config && vdf->vref)
 			s->vref = lookup_vref(s, vdf->vref);
 		}
-	if (s->type != SENSOR_VOLTAGE || !s->enabled)
-		return;
-	volt = g_new0(VoltMon, 1);
-	volt->sensor = s;
-	volt_list = g_list_append(volt_list, volt);
+	if (s->type == SENSOR_VOLTAGE && s->enabled)
+		{
+		smon = g_new0(SensorMon, 1);
+		smon->sensor = s;
+		smon->panel = &pVolt;
+		volt_list = g_list_append(volt_list, smon);
+		}
+	if (s->type == SENSOR_TEMPERATURE && can_monitor_sensor_name(s))
+		{
+		smon = g_new0(SensorMon, 1);
+		smon->sensor = s;
+		smon->panel = &pTemp;
+		temperature_list = g_list_append(temperature_list, smon);
+		}
+	if (s->type == SENSOR_FAN && can_monitor_sensor_name(s))
+		{
+		smon = g_new0(SensorMon, 1);
+		smon->sensor = s;
+		smon->panel = &pFan;
+		fan_list = g_list_append(fan_list, smon);
+		}
 	}
 
   /* Verify config of volt orders has not been munged.
@@ -1339,14 +1376,16 @@ check_volt_order(void)
 	}
 
 static void
-get_volt_list(void)
+get_sensor_monitor_lists(void)
 	{
 	GList	*list;
 
 	free_glist_and_data(&volt_list);
+	free_glist_and_data(&temperature_list);
+	free_glist_and_data(&fan_list);
 	check_volt_order();
 	for (list = sensor_list; list; list = list->next)
-		add_volt_mon((Sensor *) list->data);
+		add_sensor_monitor((Sensor *) list->data);
 
 	/* Make sure the themers can see voltage monitor.
 	*/
@@ -1370,33 +1409,60 @@ get_volt_list(void)
 			demo1_sensor->volt_order = demo1_sensor->id = 1;
 			n_volt_sensors = 2;
 			}
-		add_volt_mon(demo0_sensor);
-		add_volt_mon(demo1_sensor);
+		add_sensor_monitor(demo0_sensor);
+		add_sensor_monitor(demo1_sensor);
 		}
 	}
 
 #include "pixmaps/sensors/bg_volt.xpm"
 
 static void
-draw_volt_bezels(void)
+cb_alert_trigger(Alert *alert, SensorMon *smon)
+	{
+	AlertDecal	*ad;
+	Decal		*d;
+
+	ad = &alert->ad;
+	alert->panel = *smon->panel;
+
+	/* Make the AlertDecal show up under the sensor decal
+	*/
+	d = smon->sensor_decal;
+	if (d)
+		{
+		ad->x = d->x - 2;
+		ad->y = d->y - 2;
+		ad->w = d->w + 3;
+		ad->h = d->h + 4;
+		gkrellm_render_default_alert_decal(alert);
+		}
+	}
+
+
+static void
+draw_bezels(Panel *p, GList *smon_list, gint w, gint h, gint x_adjust)
 	{
 	GList			*list;
 	GdkImlibBorder	*b		= &bezel_style->border;
-	VoltMon			*volt;
+	SensorMon		*smon;
 	Decal			*dv;
 	gint			x;
 
 	if (!bezel_image)
 		return;
-	for (list = volt_list; list; list = list->next)
+	for (list = smon_list; list; list = list->next)
 		{
-		volt = (VoltMon *) list->data;
-		dv = volt->volt_decal;
-		x = dv->x + pixel_grub;
-		gdk_imlib_paste_image(bezel_image, pVolt->bg_pixmap,
-				x - b->left, dv->y - b->top, bezel_width, volt_mon_height);
-		gdk_imlib_paste_image(bezel_image, pVolt->pixmap,
-				x - b->left, dv->y - b->top, bezel_width, volt_mon_height);
+		smon = (SensorMon *) list->data;
+		dv = smon->sensor_decal;
+		x = dv->x + x_adjust;
+		if (w == 0)
+			w = b->left + dv->w + b->right - x_adjust;
+		if (h == 0)
+			h = b->top + b->bottom + dv->h;
+		gdk_imlib_paste_image(bezel_image, p->bg_pixmap,
+				x - b->left, dv->y - b->top, w, h);
+		gdk_imlib_paste_image(bezel_image, p->pixmap,
+				x - b->left, dv->y - b->top, w, h);
 		}
 	}
 
@@ -1405,18 +1471,18 @@ any_negative_volts(void)
 	{
 	GList		*list;
 	Sensor		*s;
-	VoltMon		*volt;
+	SensorMon		*volt;
 	gfloat		v;
 	gboolean	tmp, result = FALSE;
 
 	/* This routine can be called before any volt decals exist, but reading
 	|  voltages can trigger alerts which expect to find decals.  Hence freeze.
 	*/
-	tmp = use_threads;
-	use_threads = FALSE;	/* Need results immediately */
+	tmp = thread_data_valid;
+	thread_data_valid = FALSE;	/* Need results immediately */
 	for (list = volt_list; list; list = list->next)
 		{
-		volt = (VoltMon *) list->data;
+		volt = (SensorMon *) list->data;
 		gkrellm_freeze_alert(volt->sensor->alert);
 		s = volt->sensor->vref;
 		if (s && s->value == 0)
@@ -1429,20 +1495,20 @@ any_negative_volts(void)
 			break;
 			}
 		}
-	use_threads = tmp;
+	thread_data_valid = tmp;
 	return result;
 	}
 
-static VoltMon *
+static SensorMon *
 voltmon_first(void)
 	{
-	GList	*list;
-	VoltMon	*volt = NULL;
-	VoltMon	*vtest;
+	GList		*list;
+	SensorMon	*volt = NULL;
+	SensorMon	*vtest;
 
 	for (list = volt_list; list; list = list->next)
 		{
-		vtest = (VoltMon *) list->data;
+		vtest = (SensorMon *) list->data;
 		if (!volt)
 			volt = vtest;
 		else if (volt->sensor->volt_order > vtest->sensor->volt_order)
@@ -1451,16 +1517,16 @@ voltmon_first(void)
 	return volt;
 	}
 
-static VoltMon *
-voltmon_next(VoltMon *vcur)
+static SensorMon *
+voltmon_next(SensorMon *vcur)
 	{
-	GList	*list;
-	VoltMon	*volt = NULL;
-	VoltMon	*vtest;
+	GList		*list;
+	SensorMon	*volt = NULL;
+	SensorMon	*vtest;
 
 	for (list = volt_list; list; list = list->next)
 		{
-		vtest = (VoltMon *) list->data;
+		vtest = (SensorMon *) list->data;
 		if (vtest->sensor->volt_order > vcur->sensor->volt_order)
 			{
 			if (!volt)
@@ -1476,7 +1542,8 @@ static void
 make_volt_decals(Panel *p, Style *style)
 	{
 	GdkImlibBorder	*b		= &bezel_style->border;
-	VoltMon			*volt;
+	Sensor			*sensor;
+	SensorMon		*volt;
 	Decal			*dv, *dn;
 	TextStyle		*ts_volt, *ts_name;
 	gchar			*fmt;
@@ -1502,6 +1569,7 @@ make_volt_decals(Panel *p, Style *style)
 
 	for (volt = voltmon_first(); volt; volt = voltmon_next(volt))
 		{
+		sensor = volt->sensor;
 		if (display_mode == DIGITAL_WITH_LABELS)
 			{
 			volt->name_decal = dn = gkrellm_create_decal_text(p,
@@ -1510,18 +1578,19 @@ make_volt_decals(Panel *p, Style *style)
 				volt_name_width = dn->w;
 			}
 		dv = gkrellm_create_decal_text(p, fmt, ts_volt, style, 0, 0, 0);
-		volt->volt_decal = dv;
+		volt->sensor_decal = dv;
 		if (minus_width == 1)
 			minus_width = gdk_string_width(dv->text_style.font, "-");
 		w_volt = dv->w;			/* Same for all volt decals */
 		if (dv->h > volt_mon_height)
 			volt_mon_height = dv->h;
+
+		sensor->cb_alert = cb_alert_trigger;
+		sensor->cb_alert_data = volt;
+		gkrellm_alert_trigger_connect(sensor->alert, cb_alert_trigger, volt);
 		}
-	if (minus_width)
-		pixel_grub = (b->left > 1) ? 2 : 1;
-	else
-		pixel_grub = 0;
-	bezel_width = b->left + w_volt + b->right - pixel_grub;
+	pixel_grub = minus_width ? 1 : 0;
+	volt_bezel_width = b->left + w_volt + b->right - pixel_grub;
 	volt_mon_height += b->top + b->bottom;
 
 	/* If name decal I let bezel left border encroach into NAME_PAD space
@@ -1535,11 +1604,13 @@ make_volt_decals(Panel *p, Style *style)
 static void
 layout_volt_decals(Panel *p, Style *style)
 	{
-	VoltMon	*volt;
+	SensorMon	*volt;
 	Decal	*dv, *dn;
+	Margin	*m;
 	gint	x, y, w, c, n, cols;
 
-	w = gkrellm_chart_width() - 2 * style->margin;
+	m = gkrellm_get_style_margins(style);
+	w = gkrellm_chart_width() - m->left - m->right;
 	cols = (w + MONITOR_PAD) / (volt_mon_width + MONITOR_PAD);
 	if (cols < 1)
 		cols = 1;
@@ -1547,24 +1618,24 @@ layout_volt_decals(Panel *p, Style *style)
 	if (cols > n)
 		cols = n;;
 	volt_mon_width = w / cols;		/* spread them out */
-	x = (w - cols * volt_mon_width) / 2 + style->margin;
+	x = (w - cols * volt_mon_width) / 2 + m->left;
 		
 	gkrellm_get_top_bottom_margins(style, &y, NULL);
 	c = 0;
 	for (volt = voltmon_first(); volt; volt = voltmon_next(volt))
 		{
 		dn = volt->name_decal;
-		dv = volt->volt_decal;
+		dv = volt->sensor_decal;
 		/* Right justify the volt decal in each volt_mon field
 		*/
 		dv->x = x + (c+1) * volt_mon_width - dv->w - bezel_style->border.right;
 		if (cols > 1 && !dn)
-			dv->x -= (volt_mon_width - bezel_width) / 2;
+			dv->x -= (volt_mon_width - volt_bezel_width) / 2;
 		dv->y = y + bezel_style->border.top;
 		if (dn)
 			{
 			if (cols == 1)
-				dn->x = style->margin;
+				dn->x = m->left;
 			else
 				dn->x = dv->x - volt_name_width - NAME_PAD;
 			dn->y = y + bezel_style->border.top;
@@ -1579,24 +1650,90 @@ layout_volt_decals(Panel *p, Style *style)
 		}
 	}
 
+static gchar *
+skip_underscore(gchar *name)
+	{
+	if (*name == '_')
+		++name;
+	return name;
+	}
+
+static void
+draw_temperatures(gboolean draw_name)
+	{
+	GList		*list;
+	SensorMon	*smon;
+	Sensor		*sensor;
+	gfloat		t;
+	gchar		units;
+
+	if (!pTemp)
+		return;
+	for (list = temperature_list; list; list = list->next)
+		{
+		smon = (SensorMon *) list->data;
+		sensor = smon->sensor;
+
+		if (draw_name && smon->name_decal)
+			gkrellm_draw_decal_text(pTemp, smon->name_decal,
+					skip_underscore(sensor->name), -1);
+		if (smon->sensor_decal)
+			{
+			gkrellm_sensor_read_temperature(sensor->name, &t, &units);
+			gkrellm_sensor_draw_temperature_decal(pTemp, smon->sensor_decal,
+					t, units);
+			}
+		}
+	gkrellm_draw_panel_layers(pTemp);
+	}
+
+static void
+draw_fans(gboolean draw_name)
+	{
+	GList		*list;
+	SensorMon	*smon;
+	Sensor		*sensor;
+	gfloat		f;
+
+	if (!pFan)
+		return;
+	for (list = fan_list; list; list = list->next)
+		{
+		smon = (SensorMon *) list->data;
+		sensor = smon->sensor;
+
+		if (draw_name && smon->name_decal)
+			gkrellm_draw_decal_text(pFan, smon->name_decal,
+					skip_underscore(sensor->name), -1);
+		if (smon->sensor_decal)
+			{
+			gkrellm_sensor_read_fan(sensor->name, &f);
+			gkrellm_sensor_draw_fan_decal(pFan, smon->sensor_decal, f);
+			}
+		}
+	gkrellm_draw_panel_layers(pFan);
+	}
+
 static void
 draw_voltages(gint do_names)
 	{
 	GList		*list;
-	VoltMon		*volt;
+	SensorMon		*volt;
 	Sensor		*sensor;
 	Decal		*d;
 	gchar		*fmt, buf[32];
 	gfloat		v;
 
+	if (!pVolt)
+		return;
 	for (list = volt_list; list; list = list->next)
 		{
-		volt = (VoltMon *) list->data;
+		volt = (SensorMon *) list->data;
 		sensor = volt->sensor;
-		sensor->value_valid = FALSE;
+		sensor->value_valid = FALSE;	/* In case vref monitoring stops */
 		if (do_names && volt->name_decal)
 			gkrellm_draw_decal_text(pVolt, volt->name_decal, sensor->name, -1);
-		if ((d = volt->volt_decal) != NULL)
+		if ((d = volt->sensor_decal) != NULL)
 			{
 			if (sensor->vref && !sensor->vref->value_valid)
 				sensor_read_voltage(sensor->vref, NULL);
@@ -1615,92 +1752,263 @@ draw_voltages(gint do_names)
 	}
 
 static void
-update_voltage(void)
+update_sensors(void)
 	{
 	if (!GK.five_second_tick)
 		return;
 #if defined(GKRELLM_THREADS)
 	if (use_threads)
+		{
+		thread_data_valid = TRUE;
 		run_sensors_thread();
+		}
 #endif
+	draw_temperatures(FALSE);
+	draw_fans(FALSE);
 	draw_voltages(FALSE);
 	}
 
 static gint
-expose_event(GtkWidget *widget, GdkEventExpose *ev)
+expose_event(GtkWidget *widget, GdkEventExpose *ev, Panel *p)
 	{
-	if (widget == pVolt->drawing_area)
-		gdk_draw_pixmap(widget->window, GK.draw1_GC, pVolt->pixmap,
+	gdk_draw_pixmap(widget->window, GK.draw1_GC, p->pixmap,
 				ev->area.x, ev->area.y, ev->area.x, ev->area.y,
 				ev->area.width, ev->area.height);
 	return FALSE;
 	}
 
 static gint
-cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
+cb_panel_press(GtkWidget *widget, GdkEventButton *ev, Panel *p)
 	{
 	if (ev->button == 3)
-		gkrellm_open_config_window(mon_sensors);
+		gkrellm_open_config_window(mon_config_sensors);
 	return TRUE;
 	}
 
 static void
-cb_volt_alert_trigger(Alert *alert, Sensor *sensor)
+destroy_sensor_monitors(void)
+	{
+	gkrellm_panel_destroy(pTemp);
+	gkrellm_panel_destroy(pFan);
+	gkrellm_panel_destroy(pVolt);
+	pTemp = pFan = pVolt = NULL;
+	}
+
+static GdkImlibBorder	default_bezel_border = {1,1,1,1};
+
+static void
+assign_textstyles(GList *smon_list, TextStyle **ts_name, TextStyle **ts_sensor,
+		gchar *format)
 	{
 	GList		*list;
-	VoltMon		*volt;
-	AlertDecal	*ad;
-	Decal		*d;
+	Style		*style;
+	Sensor		*sensor;
+	SensorMon	*smon;
+	TextStyle	*ts, *ts_alt;
+	gint		w, w_name, w_sensor;
 
-	ad = &alert->ad;
-	alert->panel = pVolt;
-	for (list = volt_list; list; list = list->next)
+	style = gkrellm_meter_style(style_id);
+	ts = gkrellm_copy_textstyle(gkrellm_meter_textstyle(style_id));
+	ts_alt = gkrellm_copy_textstyle(gkrellm_meter_alt_textstyle(style_id));
+	w = gkrellm_chart_width() - style->margins.left - style->margins.right;
+	w_sensor = gdk_string_width(ts->font, format);
+	w_sensor += bezel_style->border.left + bezel_style->border.right;
+	for (list = smon_list; list; list = list->next)
 		{
-		volt = (VoltMon *) list->data;
-
-		/* Make the AlertDecal show up under the right volt decal.
-		*/
-		if (volt->sensor != sensor)
-			continue;
-		d = volt->volt_decal;
-		if (d)
+		smon = (SensorMon *)list->data;
+		sensor = smon->sensor;
+		w_name = gdk_string_width(ts_alt->font, sensor->name);
+		if (w_name + w_sensor >  w - 2)
 			{
-			ad->x = d->x - 2;
-			ad->y = d->y - 2;
-			ad->w = d->w + 3;
-			ad->h = d->h + 4;
-			gkrellm_render_default_alert_decal(alert);
+			ts->font = ts_alt->font;	/* downsize the sensor font */
+			break;
 			}
-		break;
 		}
+	*ts_name = ts_alt;		/* Caller must free these */
+	*ts_sensor = ts;
+	}
+
+static gint
+adjust_decal_positions(SensorMon *smon)
+	{
+	gint	y, d, h_pad;
+
+	h_pad = bezel_style->border.top + bezel_style->border.bottom;
+	d = smon->sensor_decal->h - smon->name_decal->h;
+	y = smon->sensor_decal->y + smon->sensor_decal->h + h_pad;
+	if (d >= 0)
+		smon->name_decal->y += (d + 1) / 2;
+	else
+		{
+		if (h_pad < -d)
+			y = smon->name_decal->y + smon->name_decal->h;
+		smon->sensor_decal->y += -d / 2;
+		}
+	return y;
 	}
 
 static void
-destroy_voltage_monitor(void)
+make_temperature_panel(GtkWidget *vbox, gint first_create)
 	{
-	if (!pVolt)
+	Sensor		*sensor;
+	SensorMon	*smon = NULL;
+	Style		*style;
+	Margin		*m;
+	Decal		*d;
+	GList		*list;
+	TextStyle	*ts_sensor, *ts_name;
+	gchar		*format;
+	gint		y;
+
+	if (!pTemp)
 		return;
-	gkrellm_panel_destroy(pVolt);
-	pVolt = NULL;
-	}
+	style = gkrellm_meter_style(style_id);
+	m = &style->margins;
+	format = units_fahrenheit ? "188.8F" : "88.8C";
+	assign_textstyles(temperature_list, &ts_name, &ts_sensor, format);
+	gkrellm_get_top_bottom_margins(style, &y, NULL);
+	y += bezel_style->border.top;
+	for (list = temperature_list; list; list = list->next)
+		{
+		smon = (SensorMon *) list->data;
+		sensor = smon->sensor;
+		d = gkrellm_create_decal_text(pTemp, format,
+				ts_sensor, style, -1, y, 0);
+		d->x = gkrellm_chart_width() - d->w - m->right - 1;
+		smon->sensor_decal = d;
 
-static GdkImlibBorder	default_bezel_border = {2,1,1,1};
-
-static void
-create_voltage_monitor(GtkWidget *vbox, gint first_create)
-	{
-	Style	*style;
-	gchar	**xpm;
-
-	get_volt_list();
-	if (!volt_list)
-		return;
+		smon->name_decal = gkrellm_create_decal_text(pTemp, sensor->name,
+					ts_name, style, -1, y, 0);
+		y = adjust_decal_positions(smon);
+		sensor->cb_alert = cb_alert_trigger;
+		sensor->cb_alert_data = smon;
+		gkrellm_alert_trigger_connect(sensor->alert, cb_alert_trigger, smon);
+		}
+	g_free(ts_name);
+	g_free(ts_sensor);
+	gkrellm_panel_configure(pTemp, NULL, style);
+	if (smon && smon->sensor_decal->y + smon->sensor_decal->h >
+		smon->name_decal->y + smon->name_decal->h - bezel_style->border.bottom
+	   )
+		gkrellm_panel_configure_add_height(pTemp, bezel_style->border.bottom);
+	gkrellm_panel_create(vbox, mon_sensors, pTemp);
+	draw_bezels(pTemp, temperature_list, 0, 0, 1);
 	if (first_create)
 		{
-		pVolt = gkrellm_panel_new0();
-		bezel_style = gkrellm_style_new0();
+		gtk_signal_connect(GTK_OBJECT(pTemp->drawing_area), "expose_event",
+				(GtkSignalFunc) expose_event, pTemp);
+		gtk_signal_connect(GTK_OBJECT(pTemp->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_panel_press, pTemp);
 		}
+	draw_temperatures(TRUE);
+	}
+
+static void
+make_fan_panel(GtkWidget *vbox, gint first_create)
+	{
+	Sensor		*sensor;
+	SensorMon	*smon = NULL;
+	Style		*style;
+	Margin		*m;
+	Decal		*d;
+	GList		*list;
+	TextStyle	*ts_sensor, *ts_name;
+	gchar		*format;
+	gint		y;
+
+	if (!pFan)
+		return;
 	style = gkrellm_meter_style(style_id);
+	m = &style->margins;
+	format = "8888";
+	assign_textstyles(temperature_list, &ts_name, &ts_sensor, format);
+	gkrellm_get_top_bottom_margins(style, &y, NULL);
+	y += bezel_style->border.top;
+	for (list = fan_list; list; list = list->next)
+		{
+		smon = (SensorMon *) list->data;
+		sensor = smon->sensor;
+		d = gkrellm_create_decal_text(pFan, format,
+				ts_sensor, style, -1, y, 0);
+		d->x = gkrellm_chart_width() - d->w - m->right - 1;
+		smon->sensor_decal = d;
+
+		smon->name_decal = gkrellm_create_decal_text(pFan, sensor->name,
+					ts_name, style, -1, y, 0);
+		y = adjust_decal_positions(smon);
+		sensor->cb_alert = cb_alert_trigger;
+		sensor->cb_alert_data = smon;
+		gkrellm_alert_trigger_connect(sensor->alert, cb_alert_trigger, smon);
+		}
+	g_free(ts_name);
+	g_free(ts_sensor);
+	gkrellm_panel_configure(pFan, NULL, style);
+	if (smon && smon->sensor_decal->y + smon->sensor_decal->h >
+		smon->name_decal->y + smon->name_decal->h - bezel_style->border.bottom
+	   )
+		gkrellm_panel_configure_add_height(pFan, bezel_style->border.bottom);
+	gkrellm_panel_create(vbox, mon_sensors, pFan);
+	draw_bezels(pFan, fan_list, 0, 0, 0);
+	if (first_create)
+		{
+		gtk_signal_connect(GTK_OBJECT(pFan->drawing_area), "expose_event",
+				(GtkSignalFunc) expose_event, pFan);
+		gtk_signal_connect(GTK_OBJECT(pFan->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_panel_press, pFan);
+		}
+	draw_fans(TRUE);
+	}
+
+static void
+make_volt_panel(GtkWidget *vbox, gint first_create)
+	{
+	Style	*style;
+
+	if (!pVolt)
+		return;
+	style = gkrellm_meter_style(style_id);
+	make_volt_decals(pVolt, style);
+	layout_volt_decals(pVolt, style);
+	
+	gkrellm_panel_configure(pVolt, NULL, style);
+
+	/* Make the bottom margin reference against the bottom volt decals
+	|  bezel image.  The volt decal height does not include the bezel so
+	|  gkrellm_panel_configure() did not account for the bezel.
+	*/
+	gkrellm_panel_configure_add_height(pVolt, bezel_style->border.bottom);
+	gkrellm_panel_create(vbox, mon_sensors, pVolt);
+
+	draw_bezels(pVolt, volt_list,
+			volt_bezel_width, volt_mon_height, pixel_grub);
+
+	if (first_create)
+		{
+		gtk_signal_connect(GTK_OBJECT(pVolt->drawing_area), "expose_event",
+				(GtkSignalFunc) expose_event, pVolt);
+		gtk_signal_connect(GTK_OBJECT(pVolt->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_panel_press, pVolt);
+		}
+	draw_voltages(TRUE);
+	}
+
+static void
+create_sensors_monitor(GtkWidget *vbox, gint first_create)
+	{
+	gchar	**xpm;
+
+	get_sensor_monitor_lists();
+	if (!volt_list && !temperature_list && !fan_list)
+		return;
+	if (volt_list && !pVolt)
+		pVolt = gkrellm_panel_new0();
+	if (temperature_list && !pTemp)
+		pTemp = gkrellm_panel_new0();
+	if (fan_list && !pFan)
+		pFan = gkrellm_panel_new0();
+	if (!bezel_style)
+		bezel_style = gkrellm_style_new0();
+
 
 	/* Here is where I define the volt panel theme image extensions.  I ask
 	|  for a theme extension image:
@@ -1717,43 +2025,24 @@ create_voltage_monitor(GtkWidget *vbox, gint first_create)
 	if (!gkrellm_set_image_border("sensors_bg_volt", bezel_image, bezel_style))
 		bezel_style->border = default_bezel_border;
 
-	make_volt_decals(pVolt, style);
-	layout_volt_decals(pVolt, style);
-	
-	gkrellm_panel_configure(pVolt, NULL, style);
-
-	/* Make the bottom margin reference against the bottom volt decals
-	|  bezel image.  The volt decal height does not include the bezel so
-	|  gkrellm_panel_configure() did not account for the bezel.
-	*/
-	pVolt->label->h_panel += bezel_style->border.bottom;
-	gkrellm_panel_create(vbox, mon_volt, pVolt);
-
-	draw_volt_bezels();
-
-	if (first_create)
-		{
-		gtk_signal_connect(GTK_OBJECT(pVolt->drawing_area), "expose_event",
-				(GtkSignalFunc) expose_event, NULL);
-		gtk_signal_connect(GTK_OBJECT(pVolt->drawing_area),
-				"button_press_event", (GtkSignalFunc) cb_panel_press, NULL);
-		}
-	draw_voltages(TRUE);
+	make_temperature_panel(vbox, first_create);
+	make_fan_panel(vbox, first_create);
+	make_volt_panel(vbox, first_create);
 	}
 
 static void
-create_voltage(GtkWidget *vbox, gint first_create)
+create_sensors(GtkWidget *vbox, gint first_create)
 	{
-	voltage_vbox = vbox;
-	create_voltage_monitor(vbox, first_create);
+	sensors_vbox = vbox;
+	create_sensors_monitor(vbox, first_create);
 	}
 
-static Monitor	monitor_voltages =
+static Monitor	monitor_sensors =
 	{
-	N_("Voltage"),		/* Voltage config handled in Sensors tab */
+	N_("sensors"),		/* Voltage config handled in Sensors tab */
 	MON_VOLTAGE,		/* Id,  0 if a plugin		*/
-	create_voltage,		/* The create function		*/
-	update_voltage,		/* The update function		*/
+	create_sensors,		/* The create function		*/
+	update_sensors,		/* The update function		*/
 	NULL,				/* The config tab create function	*/
 	NULL,				/* Voltage apply handled in sensors apply */
 
@@ -1772,16 +2061,15 @@ static Monitor	monitor_voltages =
 	};
 
 Monitor *
-init_voltage_monitor(void)
+init_sensor_monitor(void)
 	{
-	style_id = gkrellm_add_meter_style(&monitor_voltages, SENSOR_STYLE_NAME);
-	mon_volt = &monitor_voltages;
-	return &monitor_voltages;
+	style_id = gkrellm_add_meter_style(&monitor_sensors, SENSOR_STYLE_NAME);
+	mon_sensors = &monitor_sensors;
+	return &monitor_sensors;
 	}
 
-/* End of voltage monitor */
 /* =================================================================== */
-/* Unified config for sensor interface and voltage monitor	*/
+/* Config for sensors monitor	*/
 
 #define	SENSOR_CONFIG_KEYWORD	"sensor"
 
@@ -1789,28 +2077,20 @@ static void
 create_sensor_alert(Sensor *s)
 	{
 	if (s->type == SENSOR_VOLTAGE)
-		{
 		s->alert = gkrellm_alert_create(NULL, s->name,
 				_("Sensor Volt Limits"),
 				TRUE, TRUE, TRUE, 20, -20, 0.01, 0.5, 2);
-		gkrellm_alert_trigger_connect(s->alert, cb_volt_alert_trigger, s);
-		}
 	else if (s->type == SENSOR_TEMPERATURE)
-		{
 		s->alert = gkrellm_alert_create(NULL, s->name,
 				_("Sensor Temperature Limits (in displayed degree units)"),
 				TRUE, FALSE, TRUE, 300, 0, 1.0, 5.0, 1);
-		gkrellm_alert_trigger_connect(s->alert, s->cb_alert, s->cb_alert_data);
-		}
 	else if (s->type == SENSOR_FAN)
-		{
 		s->alert = gkrellm_alert_create(NULL, s->name,
 				_("Sensor Fan RPM Limits"),
 				FALSE, TRUE, TRUE, 20000, 0, 100, 1000, 0);
-		gkrellm_alert_trigger_connect(s->alert, s->cb_alert, s->cb_alert_data);
-		}
 	else
 		return;
+	gkrellm_alert_trigger_connect(s->alert, s->cb_alert, s->cb_alert_data);
 	}
 
 
@@ -1961,15 +2241,21 @@ fix_temp_alert(Sensor *s)
 static void
 apply_sensors_config(void)
 	{
-	GList	*list;
-	Sensor	*s;
-	gchar	*name;
-	gint	i, previous, tmp;
+	GList		*list;
+	Sensor		*s;
+	gchar		*name;
+	gint		i, previous, tmp;
+	gboolean	name_modified;
 
 	previous = units_fahrenheit;
-	units_fahrenheit = GTK_TOGGLE_BUTTON(fahrenheit_button)->active;
+	i = GTK_TOGGLE_BUTTON(fahrenheit_button)->active;
+	if (i != units_fahrenheit)
+		sensor_monitor_modified = TRUE;		/* To resize temp bezels */
+	units_fahrenheit = i;
 #if defined(GKRELLM_THREADS)
 	use_threads = GTK_TOGGLE_BUTTON(use_threads_button)->active;
+	if (!use_threads)
+		thread_data_valid = FALSE;
 #endif
 	for (list = sensor_list; list; list = list->next)
 		{
@@ -1986,13 +2272,17 @@ apply_sensors_config(void)
 				gtk_label_set_text(
 						GTK_LABEL(GTK_BIN(s->select_button)->child), name);
 				}
-			if (gkrellm_dup_string(&s->name, name) && s->alert)
+			name_modified = gkrellm_dup_string(&s->name, name);
+			if (name_modified)
+				sensor_monitor_modified = TRUE;
+			if (name_modified && s->alert)
 				{
-				if (s->type == SENSOR_VOLTAGE)
+				if (*s->name)
 					{
 					g_free(s->alert->name);
 					s->alert->name = g_strdup_printf("%s %s", s->name,
 							_("Voltage"));
+					gkrellm_reset_alert(s->alert);
 					}
 				else
 					{
@@ -2003,12 +2293,12 @@ apply_sensors_config(void)
 			}
 		if (s->vref_entry)
 			{
-			tmp = voltages_modified;
+			tmp = sensor_monitor_modified;
 			name = gkrellm_entry_get_text(&s->vref_entry);
 			s->vref = lookup_vref(s, name);
 			gtk_entry_set_text(GTK_ENTRY(s->vref_entry),
 						s->vref ? g_basename(s->vref->path) : "");
-			voltages_modified = tmp;
+			sensor_monitor_modified = tmp;
 			}
 		if (s->factor_spin_button)
 			s->factor = gtk_spin_button_get_value_as_float(
@@ -2050,19 +2340,19 @@ apply_sensors_config(void)
 		if (GTK_TOGGLE_BUTTON(display_mode_button[i])->active)
 			display_mode = i;
 
-	if (   correction_modified && ! voltages_modified
+	if (   correction_modified && ! sensor_monitor_modified
 		&& have_negative_volts != any_negative_volts()
 	   )
-		voltages_modified = TRUE;
-	if (display_mode != previous || voltages_modified)
+		sensor_monitor_modified = TRUE;
+	if (display_mode != previous || sensor_monitor_modified)
 		{
-		destroy_voltage_monitor();
-		create_voltage_monitor(voltage_vbox, TRUE);
+		destroy_sensor_monitors();
+		create_sensors_monitor(sensors_vbox, TRUE);
 		}
 	else if (correction_modified)
 		draw_voltages(FALSE);
 
-	voltages_modified = FALSE;
+	sensor_monitor_modified = FALSE;
 	correction_modified = FALSE;
 	}
 
@@ -2076,7 +2366,7 @@ static void
 cb_volt_enabled(GtkWidget *button, Sensor *voltage)
 	{
 	voltage->enabled = !voltage->enabled;
-	voltages_modified = TRUE;
+	sensor_monitor_modified = TRUE;
 	}
 
 static void
@@ -2084,7 +2374,7 @@ cb_volt_entry(GtkWidget *entry, Sensor *voltage)
 	{
 	gchar	*name;
 
-	voltages_modified = TRUE;
+	sensor_monitor_modified = TRUE;
 	name = gkrellm_entry_get_text(&entry);
 	if (voltage->select_button)
 		gtk_label_set_text(
@@ -2094,7 +2384,7 @@ cb_volt_entry(GtkWidget *entry, Sensor *voltage)
 static void
 cb_vref_entry(GtkWidget *entry, gpointer data)
 	{
-	voltages_modified = TRUE;
+	sensor_monitor_modified = TRUE;
 	}
 
 static void
@@ -2105,7 +2395,7 @@ cb_volt_order(GtkWidget *button, Sensor *sv)
 	Sensor			*sr;
 	gint			order, replaced_order;
 
-	voltages_modified = TRUE;
+	sensor_monitor_modified = TRUE;
 	spin = GTK_SPIN_BUTTON(sv->volt_order_spin_button);
 	order = gtk_spin_button_get_value_as_int(spin);
 	if (order == sv->volt_order)
@@ -2146,26 +2436,20 @@ cb_sensor_set_alert(GtkWidget *entry, Sensor *s)
 
 static gchar	*sensor_info_text0[]	= 
 	{
-N_("Depending on the hardware sensor implementation of your motherboard,\n"
-"there may be temperature, fan, and voltage sensors available to monitor.\n"
-"If there are temperature or fan sensors, the data for these may be\n"
-"displayed on the CPU and Proc panels.  If there are voltage sensors, the\n"
-"voltage data may be displayed on a separate panel below the Proc panel.\n"),
-
 N_("<b>\nSetup\n"),
-
-N_("On the Setup page you can enter a Sensor Label next to a temperature or\n"
-"fan Sensor File to make the data from that sensor appear on a CPU or Proc\n"
-"panel.  The label must be one of these:\n")
+N_("For temperature or fan sensors, enter a label on the Setup page next to\n"
+"a Sensor File to display it. The display will appear on a CPU or Proc panel\n"
+"if you use reserved name labels which are (lower case):\n")
 	};
 
 static gchar	*sensor_info_text1[]	= 
 	{
-N_("\nwhere the \"mb\" label is for linking to the Proc panel.\n\n"),
-N_("Voltages are displayed in their own panel so do not need a special label\n"
-"to make them appear.  They have a default label already assigned (which may\n"
-"be overridden), and are enabled separately in the Voltages tab.\n\n"),
-
+"\n",
+N_("where the \"mb\" label is for displaying on the Proc panel. If the label\n"
+"is not a reserved label, the data will be displayed on a separate panel.\n"
+"If you wish to use a reserved label with a temperature or fan display on\n"
+"the separate panel, prepend an underscore.  For example, use \"_cpu\".\n"),
+"\n",
 N_("The second function of the Setup page is to assign data scaling factors\n"
 "and offsets to the various sensors.  Some fan and voltage sensors will have\n"
 "default values, but you should review them for correctness - be sure to \n"
@@ -2176,15 +2460,21 @@ N_("The second function of the Setup page is to assign data scaling factors\n"
 "\n",
 N_("There will be a Vref entry for negative voltages if you have a sensor\n"
 "chip that uses an external voltage reference for level shifting negative\n"
-"input voltages (such as the lm80).  See the README or do a man gkrellm.\n")
+"input voltages (such as the lm80).  See the README or do a man gkrellm.\n"),
+"\n",
+N_("<b>Voltages"),
+"\n",
+N_("To display a voltage, select it on the Voltages page.  Voltages always\n"
+"have a label assigned.  The default voltage labels may be changed on the\n"
+"Setup page.\n")
 	};
 
 static gchar *sensor_name_set[4] =
 	{
-	"<b>\t\t\"cpu\" or \"mb\"",
-	"<b>\t\t\"cpu0\" or \"mb\"",
-	"<b>\t\t\"cpu0\", \"cpu1\", or \"mb\"",
-	"<b>\t\t\"cpu0\", \"cpu1\"..., or \"mb\"",
+	"<b>\t\tcpu   mb",
+	"<b>\t\tcpu0   mb",
+	"<b>\t\tcpu0   cpu1   mb",
+	"<b>\t\tcpu0   cpu1 ...   mb",
 	};
 
 static void
@@ -2213,14 +2503,14 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	gtk_table_set_col_spacings(GTK_TABLE(table), 0);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 3);
 
-	label = gtk_label_new(_("Sensor Label"));
+	label = gtk_label_new(_("Label"));
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
 
 #if !defined(__FreeBSD__)
 	label = gtk_label_new(SENSORS_DIR);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
 #endif
-	label = gtk_label_new(_("Sensor Files"));
+	label = gtk_label_new(_("Sensor File"));
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 1, 2);
 
 	label = gtk_label_new(_("Sensor Correction"));
@@ -2296,10 +2586,10 @@ create_sensors_tab(GtkWidget *tab_vbox)
 				cb_sensor_set_alert, sr, "");
 		set_alert_button_state(sr);
 		}
-	gkrellm_check_button(vbox, &fahrenheit_button, units_fahrenheit, FALSE, 2,
+	gkrellm_check_button(vbox, &fahrenheit_button, units_fahrenheit, FALSE, 0,
 		_("Display fahrenheit"));
 #if defined(GKRELLM_THREADS)
-	gkrellm_check_button(vbox, &use_threads_button, use_threads, FALSE, 2,
+	gkrellm_check_button(vbox, &use_threads_button, use_threads, FALSE, 0,
 		_("Read sensors in a thread (for slow sensor interfaces)"));
 #endif
 
@@ -2370,7 +2660,7 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	}
 
 
-static Monitor	monitor_sensors =
+static Monitor	monitor_config_sensors =
 	{
 	N_("Sensors"),				/* Name, for config tab.	*/
 	-1,					/* Id,  0 if a plugin		*/
@@ -2394,13 +2684,13 @@ static Monitor	monitor_sensors =
 	};
 
 Monitor *
-init_sensors_monitor(void)
+init_sensors_config_monitor(void)
 	{
 	if (!setup_sensor_interface() && !GK.demo)
 		return NULL;
 	sensor_list = g_list_sort(sensor_list, (GCompareFunc) strcmp_sensor_path);
-    monitor_sensors.name = _(monitor_sensors.name);
-	mon_sensors = &monitor_sensors;
-	return &monitor_sensors;
+    monitor_config_sensors.name = _(monitor_config_sensors.name);
+	mon_config_sensors = &monitor_config_sensors;
+	return &monitor_config_sensors;
 	}
 
