@@ -238,31 +238,29 @@ update_uptime(void)
 static void
 create_uptime(GtkWidget *vbox, gint first_create)
 	{
-	static Style	uptime_style;
-	TextStyle		*ts;
-	gint			w,
-					chart_width = gkrellm_chart_width();
+	Style		*style;
+	Margin		*m;
+	TextStyle	*ts;
+	gint		w,
+				chart_width = gkrellm_chart_width();
 
-	/* Give a dummy string to configure_panel() to get panel height right.
-	|  then, null out the label string.
-	*/
 	if (first_create)
 		uptime = gkrellm_panel_new0();
 
-	uptime_style = *gkrellm_meter_style(style_id);
-	uptime_style.label_position = LABEL_CENTER;
+	style = gkrellm_meter_style(style_id);
+	m = gkrellm_get_style_margins(style);
 
 	ts = gkrellm_meter_textstyle(style_id);
 
 	w = gdk_string_width(ts->font, "999d 23:99") + 2;
-	if (w > chart_width - 2 * uptime_style.margin)
-		w = chart_width - 2 * uptime_style.margin;
+	if (w > chart_width - m->left - m->right)
+		w = chart_width - m->left - m->right;
 
 	decal_uptime = gkrellm_create_decal_text(uptime, "9d 12:99",
-				ts, &uptime_style, -1, -1, w);
+				ts, style, -1, -1, w);
 	decal_uptime->x = (chart_width - decal_uptime->w) / 2;
 
-	gkrellm_panel_configure(uptime, NULL, &uptime_style);
+	gkrellm_panel_configure(uptime, NULL, style);
 	gkrellm_panel_create(vbox, mon_uptime, uptime);
 
 	if (!uptime_enabled)
