@@ -175,14 +175,34 @@ basename_in_list(GList *list, gchar *name)
 GList *
 string_in_list(GList *list, gchar *s)
 	{
+	if (!s)
+		return NULL;
 	for ( ; list; list = list->next)
 		{
-		if (strcmp(list->data, s) == 0)
+		if (!strcmp((gchar *) list->data, s))
 			return list;
 		}
 	return NULL;
 	}
 
+gint
+string_position_in_list(GList *list, gchar *s)
+	{
+	gint	i, n = -1;
+
+	if (!s)
+		return -1;
+	for (i = 0 ; list; list = list->next, ++i)
+		{
+		if (!strcmp((gchar *) list->data, s))
+			{
+			n = i;
+			break;
+			}
+		}
+	return n;
+	}
+
 #if 0
 gint
 name_in_clist(GtkWidget *clist, gchar *name)
